﻿using System.Runtime.InteropServices;
using AutoMapper;
using Escali.LicenseControl.Client.State;
using Escali.LicenseControl.Client.Utils;
using Escali.LicenseControl.Client.ClientModels;
using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Services;
using Microsoft.EntityFrameworkCore;

namespace Escali.LicenseControl.Client.ViewModels
{
    public class CustomerViewModel : IViewModel
    {

        private readonly CustomerService _customerService;
        private AlertService _alertService;

        public List<CustomerClientModel>? Customers { get; private set; }

        public CustomerViewModel(DataContext context, IMapper mapper, AlertService alertService)
        {
            _customerService = new CustomerService(context);
            _mapper = mapper;
            _alertService = alertService;
        }

        public async Task GetCustomers()
        {
            var customers = await _customerService.GetAllCustomers();
            UpdateCustomers(_mapper.Map<List<CustomerClientModel>>(customers));
        }

        public async Task<CustomerClientModel?> GetCustomerById(int id)
        {
            var customer = await _customerService.GetCustomerById(id);
            return customer != null ? _mapper.Map<CustomerClientModel>(customer) : null;
        }

        public async Task<CustomerClientModel> AddCustomer(CustomerClientModel customer)
        {
            try
            {
                var mappedCustomer = _mapper.Map<Customer>(customer);
                var res = await _customerService.AddCustomer(mappedCustomer);
                _alertService.UpdateInfoMessage(new InfoMessage("Kunde ble lagt til!", InfoMessageType.SUCCESS));

                var updatedCustomers = Customers ?? new();
                updatedCustomers.Add(_mapper.Map<CustomerClientModel>(res));

                UpdateCustomers(updatedCustomers);

                return _mapper.Map<CustomerClientModel>(res);
            }
            catch (DbUpdateException e)
            {
                _alertService.UpdateInfoMessage(new InfoMessage("Det oppstod en feil i database.", InfoMessageType.ERROR));
                return null;
            }
            catch (InvalidOperationException e)
            {
                _alertService.UpdateInfoMessage(new InfoMessage(e.Message, InfoMessageType.ERROR));
                Logger.Error(e.StackTrace);
                return null;
            }
        }

        public async Task<CustomerClientModel?> UpdateCustomer(CustomerClientModel customer)
        {
            var mappedCustomer = _mapper.Map<Customer>(customer);

            try
            {
                var res = await _customerService.UpdateCustomer(mappedCustomer);
                _alertService.UpdateInfoMessage(new InfoMessage("Kunde ble oppdatert!", InfoMessageType.SUCCESS));

                var updatedCustomers = Customers!.Select(c => c.CustomerSeq != res.CustomerSeq ? c : _mapper.Map<CustomerClientModel>(res)).ToList();
                UpdateCustomers(updatedCustomers);

                return _mapper.Map<CustomerClientModel>(res);
            }
            catch (DbUpdateException e)
            {
                _alertService.UpdateInfoMessage(new InfoMessage("Det oppstod en feil i database.", InfoMessageType.ERROR));
                return null;
            }
            catch (InvalidOperationException e)
            {
                _alertService.UpdateInfoMessage(new InfoMessage(e.Message, InfoMessageType.ERROR));
                return null;
            }
        }

        private void UpdateCustomers(List<CustomerClientModel> customers)
        {
            Customers = customers;
            OnDataChanged();
        }

    }
}