using System;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Services;
using Escali.LicenseControl.DataAccess.Tests.TestUtils;
using Xunit;

namespace Escali.LicenseControl.DataAccess.Tests.Services
{

/// <summary>
/// AgreementElementService Testing Environment
/// </summary>
public class AgreementElementServiceTest
{
    /// <summary>
    /// Creates an agreementelement
    /// </summary>
    /// <returns>AgreementElement</returns>
    private AgreementElement CreateAgreementElement()
    {
        return new AgreementElement()
        {
            Price = 950.0M,
            Discount = 50.0M,
            DateFrom = DateTime.Parse("2020-01-01"),
            DateTo = DateTime.Parse("2020-12-31"),
            AgreementSeq = 1,
            ModuleLevelSeq = 1,
            AgreementElementUpdated = DateTime.Now,
            AgreementElementCreated = DateTime.Now
        };
    }

    /// <summary>
    /// passes if agreementelement-list is empty
    /// </summary>
    [Fact]
    public async void GetAllAgreementElements_Empty()
    {
        var context = TestBase.GetContext();
        var service = new AgreementElementService(context);
        var res = await service.GetAllAgreementElements();
        Assert.Empty(res);
    }
    
    /// <summary>
    /// passses if agreementelement-list is not empty after insertion
    /// </summary>
    [Fact]
    public async void GetAllAgreementElements_NotEmpty()
    {
        var context = TestBase.GetContext();
        TestData.InsertDataForAgreementElement(context);

        var ae = CreateAgreementElement();
        context.Add(ae);
        await context.SaveChangesAsync();
        
        var service = new AgreementElementService(context);
        var res = await service.GetAllAgreementElements();
        Assert.NotEmpty(res);
    }
    
    /// <summary>
    /// passes if agreementelement is added to list and found by id in the agreementelementlist
    /// </summary>
    [Fact]
    public async void GetAgreementElementsById_Success()
    {
        var context = TestBase.GetContext();
        TestData.InsertDataForAgreementElement(context);

        var ae = CreateAgreementElement();
        context.Add(ae);
        await context.SaveChangesAsync();
        
        var service = new AgreementElementService(context);
        var res = await service.GetAgreementElementById(1);
        
        Assert.Equal(ae.DateFrom, res.DateFrom);
        Assert.NotNull(res.Agreement);
        Assert.NotNull(res.ModuleLevel);
    }

    /// <summary>
    /// passes if agreementelement is added
    /// </summary>
    [Fact]
    public async void AddAgreementElement_Success()
    {
        var context = TestBase.GetContext();
        TestData.InsertDataForAgreementElement(context);
        var service = new AgreementElementService(context);

        var ae = CreateAgreementElement();
        var res = await service.AddAgreementElement(ae);
        
        Assert.Equal(ae.Discount, res.Discount);
        
    }
    
    /// <summary>
    /// passes if added agreementelement with missing fields
    /// </summary>
    [Fact]
    public async void AddAgreementElement_FieldMissing()
    {
        var context = TestBase.GetContext();
        TestData.InsertDataForAgreementElement(context);
        var service = new AgreementElementService(context);

        var ae = CreateAgreementElement();
        
        var res = await service.AddAgreementElement(ae);
        
        Assert.Equal(ae.Discount, res.Discount);
        
    }
    
    /// <summary>
    /// passes if datefrom is after dateto, and throws InvalidOperationException
    /// </summary>
    [Fact]
    public async void AddAgreementElement_DateFromNotBeforeDateTo()
    {
        var context = TestBase.GetContext();
        TestData.InsertDataForAgreementElement(context);

        var service = new AgreementElementService(context);
        var ae = CreateAgreementElement();
        ae.DateFrom = DateTime.Parse("2021-01-01");
        var op = async () => await service.AddAgreementElement(ae);

        await Assert.ThrowsAsync<InvalidOperationException>(op);

    }

    }
}
