using System;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Services;
using Escali.LicenseControl.DataAccess.Tests.TestUtils;
using Microsoft.EntityFrameworkCore;
using Xunit;
using Xunit.Abstractions;

namespace Escali.LicenseControl.DataAccess.Tests.Services
{

/// <summary>
/// CurrencyService Testing Environment
/// </summary>
public class CurrencyServiceTest
{
    private readonly ITestOutputHelper _testOutputHelper;

    public CurrencyServiceTest(ITestOutputHelper testOutputHelper)
    {
        _testOutputHelper = testOutputHelper;
    }

    /// <summary>
    /// Creates a currency
    /// </summary>
    /// <returns>Currency</returns>
    private Currency CreateCurrency()
    {
        return new Currency() {CurrencyName = "USD"};
    }

    /// <summary>
    /// passes if currencyname already exist, and it throws InvalidOperationException, and list only has one 
    /// element
    /// </summary>
    [Fact]
    public async void AddCurrency_DuplicateName()
    {
        var context = TestBase.GetContext();
        context.Currencies.Add(CreateCurrency());
        await context.SaveChangesAsync();

        var service = new CurrencyService(context);
        var op = async () => await service.AddCurrency(CreateCurrency());

        var allCurrencies = await context.Currencies.ToListAsync();

        await Assert.ThrowsAsync<InvalidOperationException>(op);
        Assert.Single(allCurrencies);

    }
    
    /// <summary>
    /// passes if after adding currencyname is converted to upperchars
    /// </summary>
    [Fact]
    public async void AddCurrency_NameConvertedToUpperChars()
    {
        var context = TestBase.GetContext();

        var service = new CurrencyService(context);
        var res = await service.AddCurrency(new Currency() {CurrencyName = "usd"});
        
        Assert.Equal("USD", res.CurrencyName);

    }

    /// <summary>
    /// passes if trying to delete currency while it is referenced in pricelist, and 
    /// throws InvalidOperationException
    /// </summary>
    [Fact]
    public async void DeleteCurrency_ShouldNotBeDeletedWhenUsedAsFK()
    {
        var context = TestBase.GetContext();
        TestData.InsertProduct(context);
        var res = context.Currencies.Add(CreateCurrency());
        await context.SaveChangesAsync();

        context.PriceLists.Add(new PriceList()
        {
            PriceListName = "2022 USD",
            DateFrom = DateTime.Now,
            DateTo = DateTime.Now,
            CurrencySeq = res.Entity.CurrencySeq,
            ProductSeq = 1
        });

        await context.SaveChangesAsync();

        var service = new CurrencyService(context);
        var op = async () => await service.DeleteCurrency(1);

        await Assert.ThrowsAsync<InvalidOperationException>(op);
    }

    /// <summary>
    /// passes if correctly retrieved currency by currencyname
    /// </summary>
    [Fact]
    public async void GetCurrencyByName_Success()
    {
        var context = TestBase.GetContext();
        var currency = CreateCurrency();
        var currencyDb = context.Currencies.Add(currency);
        await context.SaveChangesAsync();

        var service = new CurrencyService(context);
        var res = await service.GetCurrencyByName(currency.CurrencyName);

        Assert.Equal(currencyDb.Entity.CurrencySeq, res.CurrencySeq);
    }
    
    /// <summary>
    /// passes if there does not exist a currency with the retrieved currency by currencyname
    /// </summary>
    [Fact]
    public async void GetCurrencyByName_Fail()
    {
        var context = TestBase.GetContext();

        var service = new CurrencyService(context);
        var res = await service.GetCurrencyByName("USD");

        Assert.Null(res);
    }


    }
}
