﻿using System;
using System.Threading.Tasks;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Services;
using Escali.LicenseControl.DataAccess.Tests.TestUtils;
using Xunit;

namespace Escali.LicenseControl.DataAccess.Tests.Services
{

/// <summary>
/// CustomerService Testing Environment
/// </summary>
public class CustomerServiceTest
{

    /// <summary>
    /// passes if customerlist is empty
    /// </summary>
    [Fact]
    public async void GetAllCustomers_ListIsEmpty()
    {
        var context = TestBase.GetContext();
        var customerService = new CustomerService(context);

        var res = await customerService.GetAllCustomers();

        Assert.Empty(res);

    }

    /// <summary>
    /// passes if customer-list is not empty after adding a customer
    /// </summary>
    [Fact]
    public async Task GetAllCustomers_AddedElementExist()
    {
        var context = TestBase.GetContext();
        context = TestData.InsertCustomer(context);
        var customersService = new CustomerService(context);

        var res = await Task.Run(() => customersService.GetAllCustomers());

        Assert.NotEmpty(res);
    }

    /// <summary>
    /// passes if customername already exist in customer-list and it throws InvalidOperationException
    /// </summary>
    [Fact]
    public async Task AddCustomer_CustomerNameAlreadyExist()
    {
        var context = TestBase.GetContext();
        context = TestData.InsertCustomer(context);

        var customerService = new CustomerService(context);

        var res = await Assert.ThrowsAsync<InvalidOperationException>(() => customerService.AddCustomer(new Customer()
        {
            OrganizationNumber = "111 111 111",
            CustomerName = "Customer1",
            CustomerSince = DateTime.Now
        }));
        Assert.Equal($"OrgNr or CustomerName already Exist", res.Message);
    }

    /// <summary>
    /// passes if customer does not exist and is correctly added
    /// </summary>
    [Fact]
    public async Task AddCustomer_CustomerDoesNotExist()
    {
        var context = TestBase.GetContext();
        var customerService = new CustomerService(context);

        var res = await Task.Run(() => customerService.AddCustomer(new Customer()
        {
            OrganizationNumber = "111 111 111",
            CustomerName = "Customer1",
            CustomerSince = DateTime.Now,
            Region = new Region()
            {
                Country = "Norge"
            }
        }));
        Assert.NotNull(res);
    }

    /// <summary>
    /// passes if customername and organizationname already exist in customer-list, and it throws 
    /// InvalidOperationException
    /// </summary>
    [Fact]
    public async Task UpdateCustomer_CustomerNameAndOrganizationNumberExist()
    {
        var context = TestBase.GetContext();
        context = TestData.InsertCustomers(context);

        var customerService = new CustomerService(context);

        var name = await Assert.ThrowsAsync<InvalidOperationException>(() => customerService.UpdateCustomer(new Customer()
        {
            OrganizationNumber = "111 111 111",
            CustomerName = "Customer1",
            CustomerSince = DateTime.Now
        }));

        var number = await Assert.ThrowsAsync<InvalidOperationException>(() => customerService.UpdateCustomer(new Customer()
        {
            OrganizationNumber = "111 111 111",
            CustomerName = "Customer123",
            CustomerSince = DateTime.Now
        }));

        Assert.Equal($"Kunde med navn 'Customer1' finnes allerede.", name.Message);
        Assert.Equal($"Kunde med org.nr. '111 111 111' finnes allerede.", number.Message);
    }

    /// <summary>
    /// passes if trying to update a customer that does not exist in customerlist
    /// </summary>
    [Fact]
    public async void UpdateCustomer_CustomerDoesNotExistReturnsNull()
    {
        var context = TestBase.GetContext();
        context = TestData.InsertCustomerWithRegionData(context);
        var customerService = new CustomerService(context);

        var res = customerService.UpdateCustomer(new Customer()
        {
            OrganizationNumber = "999 999 999",
            CustomerName = "999999",
            CustomerSince = DateTime.Now
        });

        //True hvis Task har kastet et unhandled exception
        //Så bra at den er true i så måte at det ikke vil skje noe dersom
        //noen prøver å oppdatere noe som ikke finnes
        Assert.True(res.IsFaulted);
    }

    /// <summary>
    /// passes if customer exist in customerlist, and is correctly updated
    /// </summary>
    [Fact]
    public async Task UpdateCustomer_CustomerDoesExistReturnsEntity()
    {
        var context = TestBase.GetContext();
        context = TestData.InsertCustomerWithRegionData(context);

        var customerService = new CustomerService(context);
        var customer = customerService.GetCustomerById(1);
        customer.Result.CustomerName = "CustomerUpdated";
        var res = await Task.Run(() => customerService.UpdateCustomer(customer.Result));

        Assert.NotNull(res);
        Assert.Equal(customer.Result.CustomerName, res.CustomerName);

    }

    /// <summary>
    /// passes if trying to deactivate a customer that does not exist, and it throws NullReferenceException
    /// </summary>
    [Fact]
    public async void DeactivateCustomer_CustomerDoesNotExist()
    {

        var context = TestBase.GetContext();
        context = TestData.InsertCustomer(context);

        var customerService = new CustomerService(context);
        var res = async () => await customerService.DeactivateCustomer(2);

        await Assert.ThrowsAsync<NullReferenceException>(res);
    }

    /// <summary>
    /// passes if trying to deactivate customer that does exist
    /// </summary>
    [Fact]
    public async Task DeactivateCustomer_CustomerExist()
    {

        var context = TestBase.GetContext();
        context = TestData.InsertCustomerWithRegionData(context);

        var customerService = new CustomerService(context);
        var res = await Task.Run(() => customerService.DeactivateCustomer(1));

        Assert.True(res.IsActive == false);

    }

    /// <summary>
    /// passes if trying to activate a customer that does not exist, and it throws 
    /// NullReferenceException
    /// </summary>
    [Fact]
    public async void ActivateCustomer_CustomerDoesNotExist()
    {

        var context = TestBase.GetContext();
        context = TestData.InsertCustomer(context);

        var customerService = new CustomerService(context);
        var res = async () => await customerService.ActivateCustomer(2);

        await Assert.ThrowsAsync<NullReferenceException>(res);
    }

    /// <summary>
    /// passes if trying to activate a customer that exist
    /// </summary>
    [Fact]
    public async Task ActivateCustomer_CustomerExist()
    {

        var context = TestBase.GetContext();
        context = TestData.InsertCustomerWithRegionData(context);

        var customerService = new CustomerService(context);
        var res = await Task.Run(() => customerService.ActivateCustomer(1));

        Assert.True(res.IsActive == true);

    }

    /// <summary>
    /// passes if region reference is found on customers in customerlist
    /// </summary>
    [Fact]
    public async Task CustomerService_GetRegionSeq()
    {
        var context = TestBase.GetContext();
        context = TestData.InsertCustomerWithRegionData(context);
        var customerService = new CustomerService(context);

        var firstEntry = await Task.Run(() => customerService.GetRegionSeq(region: context.Customers.Find(1).Region));
        var secondEntry = await Task.Run(() => customerService.GetRegionSeq(region: context.Customers.Find(2).Region));
        Assert.Equal(1, firstEntry);
        Assert.Equal(2, secondEntry);
    }

    /// <summary>
    /// passes if trying to add customer with new region gets a new regionseq
    /// </summary>
    [Fact]
    public async Task CustomerServiceCreatesNewRegion_GetRegionSeq()
    {
        var context = TestBase.GetContext();
        context = TestData.InsertCustomerWithRegionData(context);
        var customerService = new CustomerService(context);

        var res = await Task.Run(() => customerService.AddCustomer(new Customer()
        {
            CustomerName = "Ikke brukt",
            OrganizationNumber = "000 000 000",
            CustomerSince = DateTime.Now,
            Region = new Region()
            {
                Country = "Sverige",
                Area = "Stockholm"
            }
        }));

        Assert.Equal(3, res.RegionSeq);
    }
    }
}
