﻿using System;
using System.Threading.Tasks;
using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Services;
using Escali.LicenseControl.DataAccess.Tests.TestUtils;
using Microsoft.EntityFrameworkCore;
using Xunit;

namespace Escali.LicenseControl.DataAccess.Tests.Services
{

/// <summary>
/// MainSegmentService Testing Environment
/// </summary>
public class MainSegmentServiceTest
{
    /// <summary>
    /// Creates MainSegment and adds to the context
    /// </summary>
    /// <param name="context">DataContext</param>
    /// <returns>DataContext</returns>
    private async Task<DataContext> InsertData(DataContext context)
    {
        context.MainSegments.Add(new MainSegment() { MainSegmentName = "MainSegment1" });
        await context.SaveChangesAsync();
        return context;
    }

    /// <summary>
    /// passes if mainsegment-list is not empty after inserting mainsegment
    /// </summary>
    [Fact]
    public async void AddMainSegment_Successfull()
    {
        var context = TestBase.GetContext();
        await InsertData(context);

        var mainSegmentService = new MainSegmentServices(context);
        var res = mainSegmentService.GetAllMainSegments();

        Assert.NotEmpty(res.Result);
    }

    /// <summary>
    /// passes if mainsegmentname already exist, and it throws and InvalidOperationException
    /// </summary>
    [Fact]
    public async void AddMainSegment_MainSegmentNameAlreadyExistsThrowsInvalidOperationException()
    {

        // Arrange
        var context = TestBase.GetContext();
        await InsertData(context);

        var newMainSegmentWithSameNameAsAlreadyInserted = new MainSegment() { MainSegmentName = "MainSegment1" };
        await context.SaveChangesAsync();

        var mainSegmentService = new MainSegmentServices(context);

        // Act
        var op = async () => await mainSegmentService.AddMainSegment(newMainSegmentWithSameNameAsAlreadyInserted);

        // Assert
        await Assert.ThrowsAsync<InvalidOperationException>(op);

    }

    /// <summary>
    /// passes if mainsegmentlist is not empty after insertion
    /// </summary>
    [Fact]
    public async void GetAllMainSegments_IsNotEmpty()
    {

        // Arrange
        var context = TestBase.GetContext();
        await InsertData(context);

        // Act
        var mainSegmentService = new MainSegmentServices(context);
        var res = mainSegmentService.GetAllMainSegments();

        // Assert
        Assert.NotEmpty(res.Result);

    }

    /// <summary>
    /// passes if mainsegmentlist is not empty after insert, and mainsegmentlist is empty after remove
    /// </summary>
    [Fact]
    public async void GetAllMainSegments_IsEmpty()
    {

        // Arrange
        var context = TestBase.GetContext();
        await InsertData(context);

        // Act
        var mainSegmentService = new MainSegmentServices(context);
        var res = mainSegmentService.GetAllMainSegments();

        //Assert
        Assert.NotEmpty(res.Result);


        var indexZero = res.Result[0];
        await mainSegmentService.DeleteMainSegment(indexZero.MainSegmentSeq);

        var resUpdated = mainSegmentService.GetAllMainSegments();

        // Assert
        Assert.Empty(resUpdated.Result);


    }

    }
}
