using System;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Services;
using Escali.LicenseControl.DataAccess.Tests.TestUtils;
using Microsoft.EntityFrameworkCore;
using Xunit;
using Xunit.Abstractions;

namespace Escali.LicenseControl.DataAccess.Tests.Services
{

/// <summary>
/// ModuleLevelService Testing Environment
/// </summary>
public class ModuleLevelServiceTest
{
    private readonly ITestOutputHelper _testOutputHelper;

    public ModuleLevelServiceTest(ITestOutputHelper testOutputHelper)
    {
        _testOutputHelper = testOutputHelper;
    }

    /// <summary>
    /// Creates ModuleLevel
    /// </summary>
    /// <returns>ModuleLevel</returns>
    private ModuleLevel CreateModuleLevelObject()
    {
        return new ModuleLevel()
        {
            ModuleLevelName = "Stocks 1",
            ModuleLevelRestriction = "250M/100tr",
            ModuleSeq = 1
        };
    }
    
    /// <summary>
    /// passes if modulelevels-list is empty
    /// </summary>
    [Fact]
    public async void GetAllModuleLevels_IsEmpty()
    {

        // Arrange
        var context = TestBase.GetContext();

        // Act
        var service = new ModuleLevelService(context);
        var res = await service.GetAllModuleLevels();

        // Assert
        Assert.Empty(res);
        
    }
    
    /// <summary>
    /// passes if modulelevel-list is not empty after insertion, and the modulelevelname equals 
    /// the added modulelevelname
    /// </summary>
    [Fact]
    public async void GetAllModuleLevels_NotEmptyAndCorrectObject()
    {

        // Arrange
        var context = TestBase.GetContext();
        context = TestData.InsertProductAndModule(context);
        var moduleLevel = CreateModuleLevelObject();
        var addedModuleLevel = context.ModuleLevels.Add(moduleLevel);
        await context.SaveChangesAsync();

        // Act
        var service = new ModuleLevelService(context);
        var res = await service.GetAllModuleLevels();

        // Assert
        Assert.NotEmpty(res);
        Assert.Equal(moduleLevel.ModuleLevelName, res[0].ModuleLevelName);
        
    }
    
    /// <summary>
    /// passes if modulelevel with id 100 does not exist
    /// </summary>
    [Fact]
    public async void GetModuleLevelById_ModuleLevelNotExisting()
    {

        // Arrange
        var context = TestBase.GetContext();

        // Act
        var service = new ModuleLevelService(context);
        var res = await service.GetModuleLevelById(100);

        // Assert
        Assert.Null(res);
        
    }
    
    /// <summary>
    /// passes if modulelevel is inserted, and module is found by id
    /// </summary>
    [Fact]
    public async void GetModuleById_ModuleExisting()
    {

        // Arrange
        var context = TestBase.GetContext();
        context = TestData.InsertProductAndModule(context);
        context.ModuleLevels.Add(CreateModuleLevelObject());
        await context.SaveChangesAsync();

        // Act
        var moduleService = new ModuleService(context);
        var res = await moduleService.GetModuleById(1);

        // Assert
        Assert.NotNull(res);
        
    }

    /// <summary>
    /// passes if modulelevel is correctly inserted
    /// </summary>
    [Fact]
    public async void AddModuleLevel_Success()
    {

        var context = TestBase.GetContext();
        context = TestData.InsertProductAndModule(context);

        ModuleLevelService service = new ModuleLevelService(context);
        var moduleLevel = CreateModuleLevelObject();
        _testOutputHelper.WriteLine(moduleLevel.ModuleLevelRestriction);

        var res = await service.AddModuleLevel(moduleLevel);
        
        Assert.Equal(moduleLevel.ModuleLevelName, res.ModuleLevelName);
        Assert.NotNull(moduleLevel.Module);
        _testOutputHelper.WriteLine(moduleLevel.ModuleLevelRestriction);

    }
    
    /// <summary>
    /// passes if modulelevelname already exist, and it throws an InvalidOperationException
    /// </summary>
    [Fact]
    public async void AddModuleLevel_DuplicateName()
    {

        var context = TestBase.GetContext();
        context = TestData.InsertProductAndModule(context);

        ModuleLevelService service = new ModuleLevelService(context);
        var moduleLevel = CreateModuleLevelObject();

        await service.AddModuleLevel(moduleLevel);
        var op = async () => await service.AddModuleLevel(CreateModuleLevelObject());

        var ex = await Assert.ThrowsAsync<InvalidOperationException>(op);

    }

    /// <summary>
    /// passes if modulelevel is missing a reference to a module, and it throws a DbUpdateException
    /// </summary>
    [Fact]
    public async void AddModuleLevel_MissingModuleRef()
    {

        var context = TestBase.GetContext();

        ModuleLevelService service = new ModuleLevelService(context);
        var moduleLevel = new ModuleLevel()
        {
            ModuleLevelName = "Stocks 1",
            ModuleLevelRestriction = "250M/100tr"
        }; ;

        var op = async () => await service.AddModuleLevel(moduleLevel);

        await Assert.ThrowsAsync<DbUpdateException>(op);

    }

    }
}
