using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Services;
using Escali.LicenseControl.DataAccess.Tests.TestUtils;
using Microsoft.Data.Sqlite;
using Microsoft.EntityFrameworkCore;
using Xunit;
using Xunit.Abstractions;

namespace Escali.LicenseControl.DataAccess.Tests.Services
{

/// <summary>
/// ModuleService Testing Environment
/// </summary>
public class ModuleServiceTest
{
    private readonly ITestOutputHelper _testOutputHelper;

    public ModuleServiceTest(ITestOutputHelper testOutputHelper)
    {
        _testOutputHelper = testOutputHelper;
    }

    /// <summary>
    /// passes if module-list is empty
    /// </summary>
    [Fact]
    public async void GetAllModules_IsEmpty()
    {

        // Arrange
        var context = TestBase.GetContext();

        // Act
        var moduleService = new ModuleService(context);
        var res = await moduleService.GetAllModules();

        // Assert
        Assert.Empty(res);

    }

    /// <summary>
    /// passes if after insertion module-list is not empty and added module is same as in list
    /// </summary>
    [Fact]
    public async void GetAllModules_NotEmptyAndCorrectObject()
    {

        // Arrange
        var context = TestBase.GetContext();
        context = TestData.InsertProductAndModule(context);
        var moduleName = context.Modules.FindAsync(1).Result.ModuleName;

        // Act
        var moduleService = new ModuleService(context);
        var res = await moduleService.GetAllModules();

        // Assert
        Assert.NotEmpty(res);
        Assert.Equal(moduleName, res[0].ModuleName);

    }

    /// <summary>
    /// passes if module with moduleid does not exist, returned null
    /// </summary>
    [Fact]
    public async void GetModuleById_ModuleNotExisting()
    {

        // Arrange
        var context = TestBase.GetContext();

        // Act
        var moduleService = new ModuleService(context);
        var res = await moduleService.GetModuleById(100);

        // Assert
        Assert.Null(res);

    }

    /// <summary>
    /// passes if module is found by id
    /// </summary>
    [Fact]
    public async void GetModuleById_ModuleExisting()
    {

        // Arrange
        var context = TestBase.GetContext();
        context = TestData.InsertProductAndModule(context);

        // Act
        var moduleService = new ModuleService(context);
        var res = await moduleService.GetModuleById(1);

        // Assert
        Assert.NotNull(res);

    }



    /// <summary>
    /// 
    /// </summary>
    [Fact]
    public async Task AddModule_ProductIdNotExisting()
    {
        // Arrange
        var context = TestBase.GetContext();

        // Act
        var moduleService = new ModuleService(context);
        var res = Task.Run(() => moduleService.AddModule(new Module()
        {
            ModuleName = "Module1"
        }));

        //var ex = await Assert.ThrowsAsync<DbUpdateException>(op);
        //Assert.IsType<SqliteException>(ex.InnerException);
        //_testOutputHelper.WriteLine(ex.InnerException.Message);
        Assert.NotNull(res);

    }

    /// <summary>
    /// passes if trying to add module without product reference, and it throws 
    /// DbUpdateException
    /// </summary>
    [Fact]
    public async void AddModule_ProductRefMissing()
    {
        // Arrange
        var context = TestBase.GetContext();

        // Act
        var moduleService = new ModuleService(context);
        var op = async () => await moduleService.AddModule(new Module()
        {
            ModuleName = "Module1"
        });

        // Assert
        // Because product key is required in Module
        await Assert.ThrowsAsync<ArgumentNullException>(op);
    }

    /// <summary>
    /// passes if modulename already exist in module-list, and throws InvalidOperationException
    /// </summary>
    [Fact]
    public async void AddModule_DuplicateName()
    {

        // Arrange
        var context = TestBase.GetContext();
        context = TestData.InsertProductAndModule(context);

        // Act
        var moduleService = new ModuleService(context);
        
        var op = async () => await moduleService.AddModule(new Module()
        {
            ModuleName = "FirstModule",
            ProductSeq = context.Products.ToListAsync().Result.Find(p => p.ProductName == "BestProductEver").ProductSeq
        });
        
        await Assert.ThrowsAsync<InvalidOperationException>(op);

    }

    /// <summary>
    /// passes if product-list contain correct modules from 
    /// </summary>
    [Fact]
    public async void AddModule_ProductHasCorrectModule()
    {
        // Arrange
        var context = TestBase.GetContext();
        context = TestData.InsertProductAndModule(context);
        
        // Act
        ModuleService moduleService = new ModuleService(context);
        //var moduleFromDb = await moduleService.AddModule(new Module() 
        //{ 
        //    ModuleName = "NewModule", 
        //    ProductSeq = context.Products.ToListAsync().Result.Find(p => p.ProductName == "BestProductEver").ProductSeq 
        //});
        var moduleFromDb = await moduleService.GetModuleById(1);
        var bestProductEver = context.Products.ToListAsync().Result.Find(p => p.ProductName == "BestProductEver");
        

        // Assert
        Assert.Equal(moduleFromDb.ModuleSeq, bestProductEver.Modules.Find(m => m.ProductSeq == bestProductEver.ProductSeq).ModuleSeq);
        Assert.Equal(moduleFromDb.ModuleName, bestProductEver.Modules.Find(m => m.ProductSeq == bestProductEver.ProductSeq).ModuleName);

    }

    /// <summary>
    /// passes if trying to update modulename to name that already exist in module-list, and throws 
    /// InvalidOperationException
    /// </summary>
    [Fact]
    public async void UpdateModule_DuplicateNameCheck()
    {
        // Arrange
        var context = TestBase.GetContext();

        var product = context.Products.Add(TestData.CreateProduct());
        await context.SaveChangesAsync();

        context.Modules.Add(new Module()
        {
            ModuleName = "NewModule",
            ProductSeq = 1
        });
        context.Modules.Add(new Module()
        {
            ModuleName = "AnotherModule",
            ProductSeq = 1
        });
        await context.SaveChangesAsync();

        ModuleService moduleService = new ModuleService(context);

        // Act
        var anotherModule = await context.Modules.FindAsync(2);
        anotherModule.ModuleName = "NewModule";
        var op = async () => await moduleService.UpdateModule(anotherModule);

        // Assert
        await Assert.ThrowsAsync<InvalidOperationException>(op);

    }

    /// <summary>
    /// passes if product reference is correctly changed when updating product reference
    /// </summary>
    [Fact]
    public async void UpdateModule_ChangeProductForModuleSuccess()
    {
        // Arrange
        var context = TestBase.GetContext();
        context = TestData.InsertDoubleProductAndModule(context);

        ModuleService moduleService = new ModuleService(context);
        ProductService ps = new ProductService(context);
        var products = ps.GetAllProducts().Result;

        // Act
        var moduleFromDb = moduleService.GetAllModules().Result.Find(m => m.ModuleName == "FirstModule");
        // Product before updating
        Assert.Equal("BestProductEver", products.Find(m => m.ProductSeq == moduleFromDb.ProductSeq).ProductName);

        moduleFromDb!.ProductSeq = products.Find(p => p.ProductName == "AnotherProduct").ProductSeq;
        await moduleService.UpdateModule(moduleFromDb);

        moduleFromDb = moduleService.GetAllModules().Result.Find(m => m.ModuleName == "FirstModule");
        // Assert
        // Product after update
        Assert.Equal("AnotherProduct", products.Find(m => m.ProductSeq == moduleFromDb.ProductSeq).ProductName);

    }

    /// <summary>
    /// passes if trying to change module's product ref to a product that doesn't exist, and it throws 
    /// a DbUpdateException
    /// </summary>
    [Fact]
    public async void UpdateModule_ChangeProductToProductSeqNotExisting()
    {
        // Arrange
        var context = TestBase.GetContext();
        context = TestData.InsertProductAndModule(context);

        ModuleService moduleService = new ModuleService(context);

        // Act (set to non existing object)
        var moduleFromDb = moduleService.GetAllModules().Result.Find(m => m.ModuleName == "FirstModule");
        moduleFromDb.ProductSeq = 534;
        var op = async () => await moduleService.UpdateModule(moduleFromDb);

        // Assert
        await Assert.ThrowsAsync<DbUpdateException>(op);

    }

    /// <summary>
    /// passes if trying to set module's product reference to null, and it throws InvalidOperationException
    /// </summary>
    [Fact]
    public async void UpdateModule_ChangeProductSetNullShouldFail()
    {
        // Arrange
        var context = TestBase.GetContext();
        context = TestData.InsertProductAndModule(context);

        ModuleService moduleService = new ModuleService(context);

        // Act (set to null)
        var moduleFromDb = moduleService.GetAllModules().Result.Find(m => m.ModuleName == "FirstModule");
        moduleFromDb.Product = null;
        var op = async () => await moduleService.UpdateModule(moduleFromDb);

        // Assert
        await Assert.ThrowsAsync<InvalidOperationException>(op);

    }

    /// <summary>
    /// passes if moduleinactiv is set to false
    /// </summary>
    [Fact]
    public async void DeactiveModule_Success()
    {
        // Arrange
        var context = TestBase.GetContext();
        context = TestData.InsertProductAndModule(context);

        ModuleService moduleService = new ModuleService(context);

        // Act
        var moduleFromDbModuleSeq = moduleService.GetAllModules().Result.Find(m => m.ModuleName == "FirstModule").ModuleSeq;
        await moduleService.DeactivateModule(moduleFromDbModuleSeq);

        var deactivatedModule = moduleService.GetAllModules().Result.Find(m => m.ModuleName == "FirstModule");

        // Assert
        Assert.Equal(false, deactivatedModule.IsActive);

    }

    /// <summary>
    /// passes if trying to add new modulelevel with already existing modulelevelname
    /// </summary>
    [Fact]
    public async void AddModule_CheckDupliacteModuleLevelNames()
    {

        // Arrange
        var context = TestBase.GetContext();
        var product = context.Products.Add(new Product() { ProductName = "BestProductEver" });
        await context.SaveChangesAsync();

        var service = new ModuleService(context);

        // Act
        var moduleLevel = new ModuleLevel()
        {
            ModuleLevelName = "Duplicate",
            ModuleLevelRestriction = "Res"
        };

        var moduleLevels = new List<ModuleLevel>
        {
            moduleLevel,
            moduleLevel
        };

        var newModule = new Module()
        {
            ProductSeq = product.Entity.ProductSeq,
            ModuleName = "ModuleName",
            ModuleLevels = moduleLevels
        };

        // Assert
        var op = async () => await service.AddModule(newModule);

        var ex = await Assert.ThrowsAsync<InvalidOperationException>(op);
        Assert.Equal("Modulnivåer kan ikke ha samme navn.", ex.Message);

    }

    /// <summary>
    /// passes if correctly added modulelevelnames in module for insertion
    /// </summary>
    [Fact]
    public async void AddModule_ModuleLevelsNotDuplicateNames()
    {

        // Arrange
        var context = TestBase.GetContext();
        var product = context.Products.Add(new Product() { ProductName = "BestProductEver" });
        await context.SaveChangesAsync();

        var service = new ModuleService(context);

        // Act
        var moduleLevel1 = new ModuleLevel()
        {
            ModuleLevelName = "Duplicate",
            ModuleLevelRestriction = "Res"
        };

        var moduleLevel2 = new ModuleLevel()
        {
            ModuleLevelName = "Not duplicate",
            ModuleLevelRestriction = "Res"
        };

        var moduleLevels = new List<ModuleLevel>
        {
            moduleLevel1,
            moduleLevel2
        };

        var newModule = new Module()
        {
            ProductSeq = product.Entity.ProductSeq,
            ModuleName = "ModuleName",
            ModuleLevels = moduleLevels
        };

        var res = await service.AddModule(newModule);

        Assert.Equal(2, res.ModuleLevels.Count);

    }

    /// <summary>
    /// passes if inactive module(module.isActive = false) is set to true
    /// </summary>
    [Fact]
    public async void ActiveModule_Success()
    {
        // Arrange
        var context = TestBase.GetContext();
        context = TestData.InsertProductAndModule(context);

        ModuleService moduleService = new ModuleService(context);

        // Act
        var moduleFromDb = moduleService.GetAllModules().Result.Find(m => m.ModuleName == "FirstModule");
        await moduleService.DeactivateModule(moduleFromDb.ModuleSeq);
        // Checking if module is deactivated IsActive=false
        Assert.False(moduleFromDb.IsActive);

        var updatedModule = moduleService.GetAllModules().Result.Find(m => m.ModuleName == "FirstModule");
        await moduleService.ActivateModule(updatedModule.ModuleSeq);

        var activatedModule = moduleService.GetAllModules().Result.Find(m => m.ModuleName == "FirstModule");

        // Assert, IsActive=true
        Assert.True(activatedModule.IsActive);

    }

    }
}
