using System;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Services;
using Escali.LicenseControl.DataAccess.Tests.TestUtils;
using Microsoft.EntityFrameworkCore;
using Xunit;

namespace Escali.LicenseControl.DataAccess.Tests.Services
{

/// <summary>
/// PriceElementService Testing Environment
/// </summary>
public class PriceElementServiceTest
{

    /// <summary>
    /// Creates a new PriceElement
    /// </summary>
    /// <returns>PriceElement</returns>
    private PriceElement CreatePriceElement()
    {
        return new PriceElement()
        {
            Price = 950,
            ModuleLevelSeq = 1,
            PriceListSeq = 1
        };
    }

    /// <summary>
    /// passses if priceelement-price equals returned priceelement-price, 
    /// and priceelement reference to module and pricelist is not null
    /// </summary>
    [Fact]
    public async void AddPriceElement_Success()
    {
        var context = TestBase.GetContext();
        TestData.InsertDataForPriceElement(context);
        var priceElement = CreatePriceElement();

        var service = new PriceElementService(context);
        var res = await service.AddPriceElement(priceElement);
        
        Assert.Equal(priceElement.Price, res.Price);
        Assert.NotNull(priceElement.ModuleLevel);
        Assert.NotNull(priceElement.PriceList);
    }

    /// <summary>
    /// passes if priceelement-price is null, and throws DbUpdateException
    /// </summary>
    [Fact]
    public async void AddPriceElement_PriceIsNull()
    {
        var context = TestBase.GetContext();
        TestData.InsertDataForPriceElement(context);
        var priceElement = CreatePriceElement();
        priceElement.Price = null;

        var service = new PriceElementService(context);
        var op = async () => await service.AddPriceElement(priceElement);

        await Assert.ThrowsAsync<DbUpdateException>(op);
    }
    
    /// <summary>
    /// passes if trying to remove pricelist when it is referencing priceelement and
    /// throws InvalidOperationException
    /// </summary>
    [Fact]
    public async void PriceElement_DeletingFKs()
    {
        var context = TestBase.GetContext();
        TestData.InsertDataForPriceElement(context);
        var priceElement = CreatePriceElement();

        var service = new PriceElementService(context);
        await service.AddPriceElement(priceElement);

        var priceList = await context.PriceLists.FindAsync(1);
        var op = () => context.PriceLists.Remove(priceList);

        Assert.Throws<InvalidOperationException>(op);
    }
    
    /// <summary>
    /// passes if trying to add priceelement without reference to modulelevel, and
    /// throws DbUpdateException
    /// </summary>
    [Fact]    
    public async void AddPriceElement_ModuleLevelFKNotExisting()
    {
        var context = TestBase.GetContext();
        TestData.InsertProduct(context);
        TestData.InsertPriceList(context);
        var priceElement = CreatePriceElement();

        var service = new PriceElementService(context);
        var op = async () => await service.AddPriceElement(priceElement);

        await Assert.ThrowsAsync<DbUpdateException>(op);
    }
    
    /// <summary>
    /// passes when trying to set priceelement-price to null, and update the priceelement, and throws
    /// DbUpdateException
    /// </summary>
    [Fact]
    public async void UpdatePriceElement_PriceIsNull()
    {
        var context = TestBase.GetContext();
        TestData.InsertDataForPriceElement(context);
        var priceElement = CreatePriceElement();
        var priceElementDb = context.PriceElements.Add(priceElement);
        await context.SaveChangesAsync();
        
        priceElementDb.Entity.Price = null;

        var service = new PriceElementService(context);
        var op = async () => await service.UpdatePriceElement(priceElementDb.Entity);

        await Assert.ThrowsAsync<DbUpdateException>(op);
    }

    }
}
