using System;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Services;
using Escali.LicenseControl.DataAccess.Tests.TestUtils;
using Microsoft.EntityFrameworkCore;
using Xunit;
using Xunit.Abstractions;

namespace Escali.LicenseControl.DataAccess.Tests.Services
{

/// <summary>
/// PriceListService Testing Environtment
/// </summary>
public class PriceListServiceTest
{
    private readonly ITestOutputHelper _testOutputHelper;

    public PriceListServiceTest(ITestOutputHelper testOutputHelper)
    {
        _testOutputHelper = testOutputHelper;
    }

    /// <summary>
    /// Creates a new pricelist
    /// </summary>
    /// <returns>PriceList</returns>
    private PriceList CreatePriceList()
    {
        return new PriceList()
        {
            PriceListName = "2022",
            DateFrom = DateTime.Parse("2018-01-01"),
            DateTo = DateTime.Parse("2018-12-31"),
            ProductSeq = 1,
            CurrencySeq = 1
        };
    }

    /// <summary>
    /// passes if pricelist is correctly added to productlist-list
    /// </summary>
    [Fact]
    public async void AddPriceList_Success()
    {
        var context = TestBase.GetContext();
        TestData.InsertProduct(context);
        TestData.InsertCurrency(context);

        var service = new PriceListService(context);
        var res = await service.AddPriceList(new PriceList()
        {
            PriceListName = "2022",
            DateFrom = DateTime.Parse("2018-01-01"),
            DateTo = DateTime.Parse("2018-12-31"),
            ProductSeq = 1,
            CurrencySeq = 1
        });

        Assert.NotNull(res);
        Assert.NotNull(res.Currency);
        Assert.NotNull(res.Product);
        Assert.Equal("2022", res.PriceListName);

    }
    
    /// <summary>
    /// passes if inserted pricelist has empty string name
    /// </summary>
    [Fact]
    public async void AddPriceList_NameIsEmptyString()
    {
        var context = TestBase.GetContext();
        TestData.InsertProduct(context);
        TestData.InsertCurrency(context);

        var service = new PriceListService(context);
        var res = await service.AddPriceList(new PriceList()
        {
            PriceListName = string.Empty,
            DateFrom = DateTime.Parse("2018-01-01"),
            DateTo = DateTime.Parse("2018-12-31"),
            ProductSeq = 1,
            CurrencySeq = 1
        });

        // Assert.NotNull(res);
        // Assert.NotNull(res.Currency);
        // Assert.NotNull(res.Product);
        Assert.Equal("", res.PriceListName);

    }
    
    /// <summary>
    /// passes if dateto is before datefrom, and InvalidOperationException is thrown
    /// </summary>
    [Fact]
    public async void AddPriceList_DateFromNotBeforeDateTo()
    {
        var context = TestBase.GetContext();
        TestData.InsertProduct(context);
        TestData.InsertCurrency(context);

        var service = new PriceListService(context);
        var op = async () => await service.AddPriceList(new PriceList()
        {
            PriceListName = "2022",
            DateFrom = DateTime.Parse("2018-12-31"),
            DateTo = DateTime.Parse("2018-01-01"),
            ProductSeq = 1,
            CurrencySeq = 1
        });

        await Assert.ThrowsAsync<InvalidOperationException>(op);

    }
    
    /// <summary>
    /// passes if dates is not set on pricelist, and DbUpdateException is thrown
    /// </summary>
    [Fact]
    public async void AddPriceList_DatesAreNull()
    {
        var context = TestBase.GetContext();
        TestData.InsertProduct(context);
        TestData.InsertCurrency(context);

        var priceList = new PriceList()
        {
            PriceListName = "2022",
            ProductSeq = 1,
            CurrencySeq = 1
        };

        var service = new PriceListService(context);
        var op = async () => await service.AddPriceList(priceList);

        await Assert.ThrowsAsync<DbUpdateException>(op);

    }

    /// <summary>
    /// Passes if pricelist datefrom and dateto is same date, and InvalidOperationException is thrown
    /// </summary>
    [Fact]
    public async void AddPriceList_DatesAreEqual()
    {
        var context = TestBase.GetContext();
        TestData.InsertProduct(context);
        TestData.InsertCurrency(context);

        var service = new PriceListService(context);
        var op = async () => await service.AddPriceList(new PriceList()
        {
            PriceListName = "2022",
            DateFrom = DateTime.Parse("2018-01-01"),
            DateTo = DateTime.Parse("2018-01-01"),
            ProductSeq = 1,
            CurrencySeq = 1
        });

        await Assert.ThrowsAsync<InvalidOperationException>(op);

    }
    
    /// <summary>
    /// passes ip pricelist is missing a currency reference, and throws a DbUpdateException
    /// </summary>
    [Fact]
    public async void AddPriceList_MissingCurrencyFK()
    {
        var context = TestBase.GetContext();
        TestData.InsertProduct(context);
        TestData.InsertCurrency(context);

        var service = new PriceListService(context);
        var op = async () => await service.AddPriceList(new PriceList()
        {
            PriceListName = "2022",
            DateFrom = DateTime.Parse("2018-01-01"),
            DateTo = DateTime.Parse("2018-12-31"),
            ProductSeq = 1
        });

        await Assert.ThrowsAsync<DbUpdateException>(op);

    }
    
    /// <summary>
    /// passes if pricelist is correctly inserted into the pricelist-list
    /// </summary>
    [Fact]
    public async void UpdatePriceList_Success()
    {
        var context = TestBase.GetContext();
        TestData.InsertProduct(context);
        TestData.InsertCurrency(context);

        var priceList = CreatePriceList();

        var priceListDb = context.PriceLists.Add(priceList);
        await context.SaveChangesAsync();

        var service = new PriceListService(context);

        priceListDb.Entity.DateFrom = DateTime.Parse("2017-01-01");
        var updatedPriceList = await service.UpdatePriceList(priceListDb.Entity);
        
        Assert.Equal(priceList.PriceListName, updatedPriceList.PriceListName);

    }
    
    /// <summary>
    /// passes if dateFrom not before dateto, throws InvalidOperationException
    /// </summary>
    [Fact]
    public async void UpdatePriceList_DateFromNotBeforeDateTo()
    {
        var context = TestBase.GetContext();
        TestData.InsertProduct(context);
        TestData.InsertCurrency(context);

        var priceList = CreatePriceList();

        var priceListDb = context.PriceLists.Add(priceList);
        await context.SaveChangesAsync();

        var service = new PriceListService(context);

        priceListDb.Entity.DateFrom = DateTime.Parse("2019-01-01");
        var op = async () => await service.UpdatePriceList(priceListDb.Entity);

        await Assert.ThrowsAsync<InvalidOperationException>(op);

    }

    }
}
