﻿using System.Threading.Tasks;
using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Services;
using Escali.LicenseControl.DataAccess.Tests.TestUtils;
using Microsoft.EntityFrameworkCore;
using Xunit;

namespace Escali.LicenseControl.DataAccess.Tests.Services
{

/// <summary>
/// PriorityService Testing Environment
/// </summary>
public class PriorityServiceTest
{

    private async Task<DataContext> InsertData(DataContext context)
    {
        context.Priorities.Add(new Priority() { PriorityName = Priority.PriorityLevel.GOLD });
        await context.SaveChangesAsync();
        return context;
    }

    /// <summary>
    /// Inserts a priority in prioritylist
    /// passes if there exist a priority in prioritylist
    /// </summary>
    [Fact]
    public async void GetAllPriorities_IsNotEmpty()
    {

        // Arrange
        var context = TestBase.GetContext();
        await InsertData(context);

        // Act
        var priortyService = new PriorityService(context);
        var res = priortyService.GetAllPriorities();

        // Assert
        Assert.Single(res.Result);

    }

    /// <summary>
    /// passes if priority exist in prioritylist and updates priorityname
    /// </summary>
    [Fact]
    public async void UpdatePriority_Allowed()
    {

        // Arrange
        var context = TestBase.GetContext();
        await InsertData(context);

        // Act
        var priorityService = new PriorityService(context);
        var res = priorityService.GetAllPriorities();
        Priority p = res.Result[0];

        p.PriorityName = Priority.PriorityLevel.GOLD;
        await priorityService.UpdatePriority(p);
        var updateRes = priorityService.GetAllPriorities();

        // Assert
        Assert.Equal(Priority.PriorityLevel.GOLD, updateRes.Result[0].PriorityName);

    }

    /// <summary>
    /// passes if priorityname is not a string
    /// because it is enum
    /// </summary>
    [Fact]
    public async void PriorityNameIsEnum_NotEqualString()
    {
        // Arrange
        var context = TestBase.GetContext();
        await InsertData(context);

        // Act
        var priorityService = new PriorityService(context);
        var res = priorityService.GetAllPriorities();
        Priority p = res.Result[0];

        // Assert
        Assert.False(p.PriorityName.Equals("GOLD"));
        Assert.IsType<Priority.PriorityLevel>(p.PriorityName);
        Assert.Equal("GOLD", p.PriorityName.ToString());
    }

    /// <summary>
    /// passes if DbUpdateException is thrown when adding unnamed priority
    /// </summary>
    [Fact]
    public async void AddPriority_PriorityNameNotSet()
    {
        // Arrange
        var context = TestBase.GetContext();

        // Act
        Priority priority = new Priority();
        var priorityService = new PriorityService(context);
        var op = async () => await priorityService.AddPriority(priority);

        await Assert.ThrowsAsync<DbUpdateException>(op);

    }

    }
}

