using System;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Services;
using Escali.LicenseControl.DataAccess.Tests.TestUtils;
using Microsoft.EntityFrameworkCore;
using Xunit;
using Xunit.Abstractions;

namespace Escali.LicenseControl.DataAccess.Tests.Services
{

/// <summary>
/// ProductService Testing Environment
/// </summary>
public class ProductServiceTest
{
    private readonly ITestOutputHelper _testOutputHelper;

    public ProductServiceTest(ITestOutputHelper testOutputHelper)
    {
        _testOutputHelper = testOutputHelper;
    }

    /// <summary>
    /// Retrieves productlist
    /// Passes if productlist is empty 
    /// </summary>
    [Fact]
    public async void GetAllProducts_IsEmpty()
    {

        // Arrange
        var context = TestBase.GetContext();

        // Act
        var productService = new ProductService(context);
        var res = await productService.GetAllProducts();

        // Assert
        Assert.Empty(res);
        
    }

    /// <summary>
    /// Inserts product in productlist
    /// passes if productlist is not empty and the inserted product equals the product in productlist
    /// </summary>
    [Fact]
    public async void GetAllProducts_NotEmptyAndCorrectObject()
    {

        // Arrange
        var context = TestBase.GetContext();
        context = TestData.InsertProduct(context);

        // Act
        var productService = new ProductService(context);
        var res = await productService.GetAllProducts();

        // Assert
        Assert.NotEmpty(res);
        Assert.Equal("BestProductEver", res[0].ProductName);
        
    }
    
    /// <summary>
    /// passes if product does not exist in the productlist
    /// </summary>
    [Fact]
    public async void GetProductById_ProductNotExisting()
    {

        // Arrange
        var context = TestBase.GetContext();

        // Act
        var productService = new ProductService(context);
        var res = await productService.GetProductById(100);

        // Assert
        Assert.Null(res);
        
    }
    
    /// <summary>
    /// passes if product is correctly added to productlist
    /// </summary>
    [Fact]
    public async void AddProduct_Success()
    {

        // Arrange
        var context = TestBase.GetContext();
        Product product = TestData.CreateProduct();
        var productService = new ProductService(context);

        // Act
        var res = await productService.AddProduct(product);

        // Assert
        Assert.Equal(product.ProductName, res.ProductName);
        
    }
    
    /// <summary>
    /// passes if productname already exist in productlist and throws an InvalidOperationException
    /// </summary>
    [Fact]
    public async void AddProduct_ProductNameAlreadyExists()
    {

        // Arrange
        var context = TestBase.GetContext();
        var product = TestData.CreateProduct();
        var addedProduct = context.Products.Add(product);
        await context.SaveChangesAsync();
        
        var productService = new ProductService(context);

        // Act
        var op = async () => await productService.AddProduct(product);

        // Assert
        await Assert.ThrowsAsync<InvalidOperationException>(op);

    }

    /// <summary>
    /// passes if product is not existing in productlist and throws a DbUpdateConcurrencyException
    /// </summary>
    [Fact]
    public async void UpdateProduct_ProductNotExisting()
    {
        var context = TestBase.GetContext();

        var service = new ProductService(context);
        var product = new Product()
        {
            ProductSeq = 100,
            ProductName = "Fake Product"
        };

        var op = async () => await service.UpdateProduct(product);

        await Assert.ThrowsAsync<DbUpdateConcurrencyException>(op);

    }
    
    /// <summary>
    /// 
    /// </summary>
    [Fact]
    public async void UpdateProduct_SameName()
    {
        var context = TestBase.GetContext();

        var service = new ProductService(context);
        var product = new Product()
        {
            ProductName = "Fake Product"
        };
        var added = context.Products.Add(product);
        await context.SaveChangesAsync();

        var fromDb = await context.Products.FindAsync(1);
        fromDb.ProductName = "Fake Product";

        var op = async () => await service.UpdateProduct(fromDb);

        // await Assert.ThrowsAsync<DbUpdateConcurrencyException>(op);

        await op.Invoke();
    }
    
    /// <summary>
    /// passes if productname is correctly changed in the productlist
    /// </summary>
    [Fact]
    public async void UpdateProduct_NewNameSuccess()
    {
        var context = TestBase.GetContext();

        var service = new ProductService(context);
        var product = new Product()
        {
            ProductName = "Fake Product"
        };
        var added = context.Products.Add(product);
        await context.SaveChangesAsync();

        var fromDb = await context.Products.FindAsync(1);
        fromDb!.ProductName = "New";
        await context.SaveChangesAsync();

        var res = await context.Products.FindAsync(fromDb.ProductSeq);
        
        Assert.Equal("New", res.ProductName);
    }

    }
}
