﻿using System;
using System.Threading.Tasks;
using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Services;
using Escali.LicenseControl.DataAccess.Tests.TestUtils;
using Microsoft.EntityFrameworkCore;
using Xunit;

namespace Escali.LicenseControl.DataAccess.Tests.Services
{

/// <summary>
/// UserService Testing Environment
/// </summary>
public class UserServiceTest
{

    /// <summary>
    /// Inserts an User into the DataContext
    /// </summary>
    /// <param name="context">DataContext</param>
    /// <returns>DataContext</returns>
    private async Task<DataContext> InsertData(DataContext context)
    {
        context.Users.Add(new User() { UserEmail = "user@gmail.com", UserConsulentName = "Bob"});
        await context.SaveChangesAsync();
        return context;
    }

    /// <summary>
    /// Adds an User to DataContext
    /// Will pass if userlist is not empty
    /// </summary>
    [Fact]
    public async void GetAllUsers_IsNotEmpty()
    {

        // Arrange
        var context = TestBase.GetContext();
        await InsertData(context);

        // Act
        var userService = new UserService(context);
        var res = userService.GetAllUsers();

        // Assert
        Assert.NotEmpty(res.Result);

    }

    /// <summary>
    /// Adds an User to DataContext with email that already exist 
    /// Will pass if InvalidOperationException is thrown
    /// </summary>
    [Fact]
    public async void AddUser_UserEmailAlreadyExists()
    {

        // Arrange
        var context = TestBase.GetContext();
        await InsertData(context);

        var newUserWithSameEmailAsAlreadyInserted = new User() { UserEmail = "user@gmail.com" };
        await context.SaveChangesAsync();

        var userService = new UserService(context);

        // Act
        var op = async () => await userService.AddUser(newUserWithSameEmailAsAlreadyInserted);

        // Assert
        await Assert.ThrowsAsync<InvalidOperationException>(op);

    }

    /// <summary>
    /// Adds an User to DataContext.UserService.Users and updates the email
    /// Will pass if updated email equals the email in the context
    /// </summary>
    [Fact]
    public async void UpdateUser_True()
    {
        // Arrange
        var context = TestBase.GetContext();
        var user = context.Users.Add(new User() { UserEmail = "user@gmail.com", UserConsulentName = "Bob"});
        await context.SaveChangesAsync();

        // Act
        var userService = new UserService(context);
        var res = userService.GetAllUsers();

        // Assert first email
        Assert.Equal(user.Entity.UserEmail, res.Result[0].UserEmail);

        // Act
        user.Entity.UserEmail = "userUpdate@gmail.com";
        context.Users.Update(user.Entity);
        await userService.UpdateUser(user.Entity);

        res = userService.GetAllUsers();

        // Assert changed email
        Assert.Equal("userUpdate@gmail.com", res.Result[0].UserEmail);

    }

    }
}
