using System;
using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;

namespace Escali.LicenseControl.DataAccess.Tests.TestUtils
{
/// <summary>
/// Reusable methods for inserting information into tests
/// </summary>
public class TestData
{

    /// <summary>
    /// Creates a Product
    /// </summary>
    /// <returns>new Product</returns>
    public static Product CreateProduct()
    {
        return new Product() { ProductName = "BestProductEver" };
    }

    /// <summary>
    /// Inserts Product into the context
    /// Used for inserting product into the test environment
    /// </summary>
    /// <param name="context">DataContext</param>
    /// <returns>DataContext</returns>
    public static DataContext InsertProduct(DataContext context)
    {
        context.Products.Add(CreateProduct());
        context.SaveChanges();
        return context;
    }

    /// <summary>
    /// Inserts Product and Module into the context
    /// Used for inserting product and module into the test environment
    /// </summary>
    /// <param name="context">DataContext</param>
    /// <returns>DataContext</returns>
    public static DataContext InsertProductAndModule(DataContext context)
    {
        var product = context.Products.Add(new Product() { ProductName = "BestProductEver" });
        context.Modules.Add(new Module() { ModuleName = "FirstModule", Product = product.Entity });
        context.SaveChanges();
        return context;
    }

    /// <summary>
    /// Inserts two Product and Module into the context
    /// Used for inserting product and module into the test environment
    /// </summary>
    /// <param name="context">DataContext</param>
    /// <returns>DataContext</returns>
    public static DataContext InsertDoubleProductAndModule(DataContext context)
    {
        var product = context.Products.Add(new Product() { ProductName = "BestProductEver" });
        context.Products.Add(new Product() { ProductName = "AnotherProduct" });
        context.Modules.Add(new Module() { ModuleName = "FirstModule", Product = product.Entity });
        context.SaveChanges();
        return context;
    }

    /// <summary>
    /// Adds a ModuleLevel to an Module with a FK
    /// </summary>
    /// <param name="context">DataContext</param>
    /// <returns>DataContext</returns>
    public static DataContext InsertModuleLevelAndFKs(DataContext context)
    {
        context = InsertProductAndModule(context);
        context.ModuleLevels.Add(new ModuleLevel()
        {
            ModuleLevelName = "Stock and fond 1",
            ModuleLevelRestriction = "250M/100tr",
            ModuleSeq = 1,
        });
        context.SaveChanges();
        return context;
    }

    /// <summary>
    /// Inserts Currency into DataContext
    /// </summary>
    /// <param name="context">DataContext</param>
    /// <returns>DataContext</returns>
    public static DataContext InsertCurrency(DataContext context)
    {
        context.Currencies.Add(new Currency() { CurrencyName = "USD" });
        context.SaveChanges();
        return context;
    }

    /// <summary>
    /// Inserts a PriceList into DataContext
    /// Product must be inserted before this method is called
    /// </summary>
    /// <param name="context">DataContext</param>
    /// <returns>DataContext</returns>
    public static DataContext InsertPriceList(DataContext context)
    {
        context = InsertCurrency(context);
        context.PriceLists.Add(new PriceList()
        {
            PriceListName = "2022",
            DateFrom = DateTime.Parse("2017-01-01"),
            DateTo = DateTime.Parse("2017-12-31"),
            CurrencySeq = 1,
            ProductSeq = 1
        });
        context.SaveChanges();
        return context;
    }

    /// <summary>
    /// 
    /// </summary>
    /// <param name="context"></param>
    /// <returns></returns>
    public static DataContext InsertDataForPriceElement(DataContext context)
    {
        context = InsertModuleLevelAndFKs(context);
        context = InsertPriceList(context);
        return context;
    }

    /// <summary>
    /// Inserting customer into DB (only non nullable field defined)
    /// </summary>
    /// <param name="context">DataContext</param>
    /// <returns>DataContext</returns>
    public static DataContext InsertCustomer(DataContext context)
    {
        context.Customers.Add(new Customer
        {
            OrganizationNumber = "123 456 789",
            CustomerName = "Customer1",
            CustomerSince = DateTime.Now
        });
        context.SaveChanges();
        return context;
    }

    /// <summary>
    /// Insert two Customers into DataContext
    /// </summary>
    /// <param name="context">DataContext</param>
    /// <returns>DataContext</returns>
    public static DataContext InsertCustomers(DataContext context)
    {
        context.Customers.Add(new Customer
        {
            OrganizationNumber = "123 456 789",
            CustomerName = "Customer1",
            CustomerSince = DateTime.Now
        });
        context.Customers.Add(new Customer
        {
            OrganizationNumber = "111 111 111",
            CustomerName = "Customer2",
            CustomerSince = DateTime.Now
        });
        context.SaveChanges();
        return context;
    }

    /// <summary>
    /// Inserts two Customer combined with region data into the DataContext
    /// </summary>
    /// <param name="context">DataContext</param>
    /// <returns>DataContext</returns>
    public static DataContext InsertCustomerWithRegionData(DataContext context)
    {
        context.Customers.Add(new Customer
        {
            OrganizationNumber = "123 456 789",
            CustomerName = "Customer1",
            CustomerSince = DateTime.Now,
            Region = new Region()
            {
                Country = "Norge",
                Area = "Bergen"
            }
        });
        context.Customers.Add(new Customer
        {
            OrganizationNumber = "888 888 888",
            CustomerName = "CustomerTest",
            CustomerSince = DateTime.Now,
            Region = new Region()
            {
                Country = "Norge"
            }
        });
        context.SaveChanges();
        return context;
    }

    /// <summary>
    /// Combined insertion for Currency, Product and Customer into DataContext
    /// for broader test option
    /// </summary>
    /// <param name="context">DataContext</param>
    /// <returns>DataContext</returns>
    public static DataContext InsertDataForAgreement(DataContext context)
    {
        context = InsertCurrency(context);
        context = InsertProduct(context);
        context = InsertCustomer(context);
        return context;
    }

    /// <summary>
    /// Uses InsertDataForAgreement
    /// Then add agreement with references to a currency, a product and a customer
    /// Also adds a Module and a modulelevel
    /// </summary>
    /// <param name="context">DataContext</param>
    /// <returns>DataContext</returns>
    public static DataContext InsertDataForAgreementElement(DataContext context)
    {
        InsertDataForAgreement(context);
        context.Agreements.Add(new Agreement()
        {
            AgreementName = "Escali Financials",
            AgreementCreated = DateTime.Now,
            CurrencySeq = 1,
            ProductSeq = 1,
            CustomerSeq = 1,
            AgreementUpdated = DateTime.Now
        });
        context.SaveChanges();
        context.Modules.Add(new Module() { ModuleName = "Module1", ProductSeq = 1 });
        context.SaveChanges();
        context.ModuleLevels.Add(new ModuleLevel()
        {
            ModuleLevelName = "Stock and fond 1",
            ModuleLevelRestriction = "250M/100tr",
            ModuleSeq = 1,
        });
        context.SaveChanges();

        return context;
    }

    }

}

