using Escali.LicenseControl.DataAccess.Models;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;

namespace Escali.LicenseControl.DataAccess.Data
{

public class DataContext : DbContext
{
    
    // TODO Divide into multiple files

    public DbSet<Product> Products { get; set; }
    public DbSet<Module> Modules { get; set; }
    public DbSet<ModuleLevel> ModuleLevels { get; set; }
    public DbSet<PriceElement> PriceElements { get; set; }
    public DbSet<PriceList> PriceLists { get; set; }
    public DbSet<Customer> Customers { get; set; }
    public DbSet<Priority> Priorities { get; set; }
    public DbSet<Segment> Segments { get; set; }
    public DbSet<MainSegment> MainSegments { get; set; }
    public DbSet<User> Users { get; set; }
    public DbSet<Region> Regions { get; set; }
    public DbSet<Currency> Currencies { get; set; }
    public DbSet<AgreementElement> AgreementElements { get; set; }
    public DbSet<Agreement> Agreements { get; set; }

    public DataContext(DbContextOptions<DataContext> options) : base(options) {}

    protected override void OnModelCreating(ModelBuilder modelBuilder)
    {
        // OnModelCreatingPartial(modelBuilder);
        
        // Default values
        modelBuilder.Entity<ModuleLevel>()
            .Property(m => m.IsActive)
            .HasDefaultValue(true);
        
        modelBuilder.Entity<Module>()
            .Property(m => m.IsActive)
            .HasDefaultValue(true);
        
        modelBuilder.Entity<PriceElement>()
            .Property(p => p.IsActive)
            .HasDefaultValue(true);
        
        modelBuilder.Entity<PriceList>()
            .Property(p => p.IsActive)
            .HasDefaultValue(true);

        modelBuilder.Entity<Customer>()
            .Property(c => c.IsActive)
            .HasDefaultValue(true);

        modelBuilder.Entity<AgreementElement>()
            .Property(a => a.IsActive)
            .HasDefaultValue(true);
        
        modelBuilder.Entity<AgreementElement>()
            .Property(a => a.IsQuotation)
            .HasDefaultValue(false);
        
        modelBuilder.Entity<AgreementElement>()
            .Property(a => a.KeepPriceOnNextPricelist)
            .HasDefaultValue(false);

        modelBuilder.Entity<Agreement>()
            .Property(a => a.IsActive)
            .HasDefaultValue(true);

        modelBuilder.Entity<Priority>()
            .Property(p => p.PriorityName)
            .HasDefaultValue(null);

        modelBuilder.Entity<Customer>()
            .Property(c => c.CustomerSince)
            .HasDefaultValueSql("getdate()");
        
        modelBuilder.Entity<Agreement>()
            .Property(a => a.AgreementCreated)
            .HasDefaultValueSql("getdate()");
        
        modelBuilder.Entity<AgreementElement>()
            .Property(ae => ae.AgreementElementCreated)
            .HasDefaultValueSql("getdate()");

        // Enum convert
        modelBuilder.Entity<Priority>()
            .Property(p => p.PriorityName)
            .HasConversion<string>();
        
        // Delete behaviour
        modelBuilder.Entity<Product>()
            .HasMany<PriceList>(p => p.PriceLists)
            .WithOne(pl => pl.Product)
            .OnDelete(DeleteBehavior.Restrict);
        
        modelBuilder.Entity<Product>()
            .HasMany<Agreement>(p => p.Agreements)
            .WithOne(a => a.Product)
            .OnDelete(DeleteBehavior.Restrict);
        
        modelBuilder.Entity<Product>()
            .HasMany<Module>(p => p.Modules)
            .WithOne(m => m.Product)
            .OnDelete(DeleteBehavior.Restrict);
        
        modelBuilder.Entity<Module>()
            .HasMany<ModuleLevel>(m => m.ModuleLevels)
            .WithOne(m => m.Module)
            .OnDelete(DeleteBehavior.Restrict);
        
        modelBuilder.Entity<ModuleLevel>()
            .HasMany<PriceElement>(m => m.PriceElements)
            .WithOne(p => p.ModuleLevel)
            .OnDelete(DeleteBehavior.Restrict);
        
        modelBuilder.Entity<PriceList>()
            .HasMany<PriceElement>(m => m.PriceElements)
            .WithOne(p => p.PriceList)
            .OnDelete(DeleteBehavior.Restrict);
        
        modelBuilder.Entity<Currency>()
            .HasMany<PriceList>()
            .WithOne(p => p.Currency)
            .OnDelete(DeleteBehavior.Restrict);
        
        modelBuilder.Entity<Currency>()
            .HasMany<Agreement>()
            .WithOne(a => a.Currency)
            .OnDelete(DeleteBehavior.Restrict);
        
        modelBuilder.Entity<Agreement>()
            .HasMany<AgreementElement>(a => a.AgreementElements)
            .WithOne(a => a.Agreement)
            .OnDelete(DeleteBehavior.Restrict);
        
        modelBuilder.Entity<ModuleLevel>()
            .HasMany<AgreementElement>(m => m.AgreementElements)
            .WithOne(a => a.ModuleLevel)
            .OnDelete(DeleteBehavior.Restrict);
        
        modelBuilder.Entity<Customer>()
            .HasMany<Agreement>(c => c.Agreements)
            .WithOne(a => a.Customer)
            .OnDelete(DeleteBehavior.Restrict);

        modelBuilder.Entity<Segment>()
            .HasMany<Customer>()
            .WithOne(c => c.Segment)
            .OnDelete(DeleteBehavior.SetNull);

        modelBuilder.Entity<MainSegment>()
            .HasMany<Segment>(ms => ms.Segments)
            .WithOne(s => s.MainSegment)
            .OnDelete(DeleteBehavior.SetNull);

        modelBuilder.Entity<Priority>()
            .HasMany<Customer>()
            .WithOne(c => c.Priority)
            .OnDelete(DeleteBehavior.SetNull);


        modelBuilder.Entity<User>()
            .HasMany<Customer>(u => u.Customers)
            .WithOne(c => c.User)
            .OnDelete(DeleteBehavior.SetNull);


        modelBuilder.Entity<Region>()
            .HasMany<Customer>()
            .WithOne(c => c.Region)
            .OnDelete(DeleteBehavior.SetNull);

        // Unique attributes
        modelBuilder.Entity<Product>()
            .HasIndex(p => p.ProductName)
            .IsUnique();

        modelBuilder.Entity<PriceList>()
            .HasIndex(p => p.PriceListName)
            .IsUnique();

        modelBuilder.Entity<Currency>()
            .HasIndex(c => c.CurrencyName)
            .IsUnique();

        modelBuilder.Entity<Customer>()
            .HasIndex(c => c.OrganizationNumber)
            .IsUnique();
        
        modelBuilder.Entity<Customer>()
            .HasIndex(c => c.CustomerName)
            .IsUnique();
        
        modelBuilder.Entity<User>()
            .HasIndex(u => u.UserEmail)
            .IsUnique();
        
        modelBuilder.Entity<MainSegment>()
            .HasIndex(ms => ms.MainSegmentName)
            .IsUnique();
        
        modelBuilder.Entity<Segment>()
            .HasIndex(s => s.SegmentName)
            .IsUnique();

        // TODO add unique names for other entities here...

    }

        // partial void OnModelCreatingPartial(ModelBuilder modelBuilder);
    }
}
