using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Utils;
using Microsoft.EntityFrameworkCore;

namespace Escali.LicenseControl.DataAccess.Services
{

/// <summary>
/// AgreementElementService class inserts and updates AgreementElement in the Database
/// </summary>
public class AgreementElementService
{
    
    private readonly DataContext _db;

    public AgreementElementService(DataContext db)
    {
        _db = db;
    }

    /// <summary>
    /// Read all AgreementElements from database
    /// </summary>
    /// <returns> A List of AgreementElements </returns>
    public async Task<List<AgreementElement>> GetAllAgreementElements()
    {
        var res = await _db.AgreementElements
            .Include(ae => ae.Agreement)
            .ThenInclude(a => a.Currency)
            .Include(ae => ae.ModuleLevel)
            .ToListAsync();
        return res;
    }

    /// <summary>
    /// Read one AgreementElement with id from database
    /// </summary>
    /// <param name="id"></param>
    /// <returns> AgreementElement with corresponding id </returns>
    /// <returns> Null if AgreementElement with id is not found</returns>
    public async Task<AgreementElement> GetAgreementElementById(int id)
    {
        var res = await _db.AgreementElements.FindAsync(id);
        return res;
    }

    /// <summary>
    /// Adds a AgreementElement to the database
    /// </summary>
    /// <param name="agreementElement"></param>
    /// <returns> Added AgreementElement object </returns>
    /// <exception cref="InvalidOperationException">DateFrom isn't before DateTo</exception>
    /// <exception cref="DbUpdateException">Database error</exception>
    public async Task<AgreementElement> AddAgreementElement(AgreementElement agreementElement)
    {
        if (agreementElement.DateFrom != null && agreementElement.DateTo != null)
            if (!ServiceUtils.DateFromIsBeforeDateTo(agreementElement.DateFrom.Value, agreementElement.DateTo.Value))
                throw new InvalidOperationException("DateFrom is not before DateTo.");
        
        agreementElement.AgreementElementUpdated = DateTime.Now;
        agreementElement.Discount ??= 0;
        
        var res = _db.AgreementElements.Add(agreementElement);
        await _db.SaveChangesAsync();
        var returned = res.Entity;
        returned.Agreement = await _db.Agreements.FindAsync(returned.AgreementSeq) ?? new Agreement();
        
        returned.ModuleLevel = await _db.ModuleLevels.FindAsync(returned.ModuleLevelSeq) ?? new ModuleLevel();
        return returned;
    }

    /// <summary>
    /// Updates changes on a AgreementElement in the database
    /// </summary>
    /// <param name="agreementElement"></param>
    /// <returns> Updated AgreementElement object </returns>
    /// <exception cref="InvalidOperationException">DateFrom isn't before DateTo</exception>
    /// <exception cref="DbUpdateException">Database error</exception>
    /// <exception cref="DbUpdateConcurrencyException">Id doesn't exist</exception>
    public async Task<AgreementElement> UpdateAgreementElement(AgreementElement agreementElement)
    {
        agreementElement.AgreementElementUpdated = DateTime.Now;
        
        if (agreementElement.DateFrom != null && agreementElement.DateTo != null)
            if (!ServiceUtils.DateFromIsBeforeDateTo(agreementElement.DateFrom.Value, agreementElement.DateTo.Value))
                throw new InvalidOperationException("DateFrom is not before DateTo.");
        
        var res = _db.AgreementElements.Update(agreementElement);
        await _db.SaveChangesAsync();
        return res.Entity;
    }

    /// <summary>
    /// Deactivates a AgreementElement by setting IsActive attribute to false
    /// </summary>
    /// <param name="id"></param>
    /// <returns> Updated AgreementElement </returns>
    /// <exception cref="NullReferenceException"> Thrown if id does not match a AgreementElement id in the database </exception>
    public async Task<AgreementElement> DeactivateAgreementElement(int id)
    {
        var agreementElement = await _db.AgreementElements.FindAsync(id);
        if (agreementElement == null)
            throw new NullReferenceException($"AgreementElement with id {id} doesn't exist.");
        agreementElement.IsActive = false;
        var res = await UpdateAgreementElement(agreementElement);
        return res;
    }

    /// <summary>
    /// Activates a AgreementElement by setting IsActive attribute to true
    /// </summary>
    /// <param name="id"></param>
    /// <returns> Updated AgreementElement </returns>
    /// <exception cref="NullReferenceException"> Thrown if id does not match a AgreementElement id in the database </exception>
    public async Task<AgreementElement> ActivateAgreementElement(int id)
    {
        var agreementElement = await _db.AgreementElements.FindAsync(id);
        if (agreementElement == null)
            throw new NullReferenceException($"AgreementElement with id {id} doesn't exist.");
        agreementElement.IsActive = true;
        var res = await UpdateAgreementElement(agreementElement);
        return res;
    }

    }
}
