using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Utils;
using Microsoft.EntityFrameworkCore;

namespace Escali.LicenseControl.DataAccess.Services
{

/// <summary>
/// AgreementService class inserts and updates Agreement in the Database
/// </summary>
public class AgreementService
{
    
    private readonly DataContext _db;

    public AgreementService(DataContext db)
    {
        _db = db;
    }

    /// <summary>
    /// Read all Agreements from database
    /// </summary>
    /// <returns> A List of Agreements </returns>
    public async Task<List<Agreement>> GetAllAgreements()
    {
        var res = await _db.Agreements
            .Include(a => a.Currency)
            .ToListAsync();
        return res;
    }

    /// <summary>
    /// Read one Agreement with id from database
    /// </summary>
    /// <param name="id"></param>
    /// <returns> Agreement with corresponding id </returns>
    /// <returns> Null if Agreement with id is not found</returns>
    public async Task<Agreement> GetAgreementById(int id)
    {
        var res = await _db.Agreements.FindAsync(id);
        return res;
    }

    /// <summary>
    /// Adds a Agreement to the database
    /// </summary>
    /// <param name="agreement"></param>
    /// <returns> Added Agreement object </returns>
    /// <exception cref="DbUpdateException">Database error</exception>
    public async Task<Agreement> AddAgreement(Agreement agreement)
    {       
        if (_db.Agreements.Any(a => a.AgreementName == agreement.AgreementName))
            throw new InvalidOperationException($"Unique name violation.");
        
        agreement.AgreementUpdated = DateTime.Now;
        agreement.AgreementCreated = DateTime.Now;
        
        var res = _db.Agreements.Add(agreement);
        await _db.SaveChangesAsync();
        return res.Entity;
    }

    /// <summary>
    /// Updates changes on a Agreement in the database
    /// </summary>
    /// <param name="agreement"></param>
    /// <returns> Updated Agreement object </returns>
    /// <exception cref="DbUpdateException">Database error</exception>
    /// <exception cref="DbUpdateConcurrencyException">Id doesn't exist</exception>
    public async Task<Agreement> UpdateAgreement(Agreement agreement)
    {
        if (_db.Agreements.Any(a => a.AgreementName == agreement.AgreementName && a.AgreementSeq != agreement.AgreementSeq))
            throw new InvalidOperationException($"Unique name violation.");
        
        agreement.AgreementUpdated = DateTime.Now;
        
        var res = _db.Agreements.Update(agreement);
        await _db.SaveChangesAsync();
        return res.Entity;
    }

    /// <summary>
    /// Deactivates a Agreement by setting IsActive attribute to false
    /// </summary>
    /// <param name="id"></param>
    /// <returns> Updated Agreement </returns>
    /// <exception cref="NullReferenceException"> Thrown if id does not match a Agreement id in the database </exception>
    public async Task<Agreement> DeactivateAgreement(int id)
    {
        var agreement = await _db.Agreements.FindAsync(id);
        if (agreement == null)
            throw new NullReferenceException($"Agreement with id {id} doesn't exist.");
        agreement.IsActive = false;
        var res = await UpdateAgreement(agreement);
        return res;
    }

    /// <summary>
    /// Activates a Agreement by setting IsActive attribute to true
    /// </summary>
    /// <param name="id"></param>
    /// <returns> Updated Agreement </returns>
    /// <exception cref="NullReferenceException"> Thrown if id does not match a Agreement id in the database </exception>
    public async Task<Agreement> ActivateAgreement(int id)
    {
        var agreement = await _db.Agreements.FindAsync(id);
        if (agreement == null)
            throw new NullReferenceException($"Agreement with id {id} doesn't exist.");
        agreement.IsActive = true;
        var res = await UpdateAgreement(agreement);
        return res;
    }

    }
}
