using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Microsoft.EntityFrameworkCore;

namespace Escali.LicenseControl.DataAccess.Services
{

/// <summary>
/// CurrencyService class inserts and updates Currency in the Database
/// </summary>
public class CurrencyService
{
    
    private readonly DataContext _db;

    public CurrencyService(DataContext db)
    {
        _db = db;
    }

    /// <summary>
    /// Read all Currencies from database
    /// </summary>
    /// <returns> A List of Currencies </returns>
    public async Task<List<Currency>> GetAllCurrencies()
    {
        var res = await _db.Currencies.ToListAsync();
        return res;
    }

    /// <summary>
    /// Read one Currency with id from database
    /// </summary>
    /// <param name="id"></param>
    /// <returns> Currency with corresponding id </returns>
    /// <returns> Null if Currency with id is not found</returns>
    public async Task<Currency> GetCurrencyById(int id)
    {
        var res = await _db.Currencies.FindAsync(id);
        return res;
    }

    /// <summary>
    /// Read one Currency with name from database
    /// </summary>
    /// <param name="name"></param>
    /// <returns> Currency with matching name </returns>
    /// <returns> Returns null if entity not found </returns>
    public async Task<Currency> GetCurrencyByName(string name)
    {
        try
        {
            return await (from c in _db.Currencies
                where c.CurrencyName == name
                select c).FirstAsync();
        }
        catch (InvalidOperationException e)
        {
            return null;
        }
    }

    /// <summary>
    /// Adds a Currency to the database
    /// </summary>
    /// <param name="currency"></param>
    /// <returns> Added Currency object </returns>
    /// <exception cref="DbUpdateException">Database error</exception>
    public async Task<Currency> AddCurrency(Currency currency)
    {
        currency.CurrencyName = currency.CurrencyName.ToUpper();

        if (_db.Currencies.Any(c => c.CurrencyName == currency.CurrencyName))
            throw new InvalidOperationException($"Unique name violation.");
        
        var res = _db.Currencies.Add(currency);
        await _db.SaveChangesAsync();
        return res.Entity;
    }

    /// <summary>
    /// Updates changes on a Currency in the database
    /// </summary>
    /// <param name="currency"></param>
    /// <returns> Updated Currency object </returns>
    /// <exception cref="DbUpdateException">Database error</exception>
    /// <exception cref="DbUpdateConcurrencyException">Id doesn't exist</exception>
    public async Task<Currency> UpdateCurrency(Currency currency)
    {
        currency.CurrencyName = currency.CurrencyName.ToUpper();
        
        if (_db.Currencies.Any(c => c.CurrencyName == currency.CurrencyName && c.CurrencySeq != currency.CurrencySeq))
            throw new InvalidOperationException($"Unique name violation.");
        
        var res = _db.Currencies.Update(currency);
        await _db.SaveChangesAsync();
        return res.Entity;
    }

    /// <summary>
    /// Deletes a currency with id in the database
    /// </summary>
    /// <param name="id"></param>
    /// <returns></returns>
    /// <exception cref="NullReferenceException"> Thrown if id does not match Currency id in database </exception>
    public async Task DeleteCurrency(int id)
    {
        var currency = await _db.Currencies.FindAsync(id);
        if (currency == null) throw new NullReferenceException($"Currency with id {id} doesn't exist.");
        _db.Currencies.Remove(currency);
    }

    }
}
