﻿using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Utils;
using Escali.LicenseControl.DataAccess.Services;
using Microsoft.EntityFrameworkCore;

namespace Escali.LicenseControl.DataAccess.Services
{

/// <summary>
/// CustomerService class inserts and updates Customer in the Database
/// </summary>
public class CustomerService
{
    private readonly DataContext _db;

    public CustomerService(DataContext db)
    {
        _db = db;
    }

    /// <summary>
    /// Read all Customers from database
    /// </summary>
    /// <returns> A List of Customers </returns>
    public async Task<List<Customer>> GetAllCustomers(int? id = null)
    {
        var res = await _db.Customers
            .Where(c => id == null || c.CustomerSeq == id)
            .Include(c => c.User)
            .Include(c => c.Region)
            .Include(c => c.Priority)
            .Include(c => c.Segment)
            .ThenInclude(s => s.MainSegment)
            .Include(c => c.Agreements)
            .AsSplitQuery()
            .ToListAsync();
        return res;
    }

    // /// <summary>
    // /// Retrieves all Customers from database including referenced tables
    // /// </summary>
    // /// <returns> A List of Customers</returns>
    // public async Task<List<Customer>> GetAllCustomerWithChildren(int? id = null)
    // {
    //     var res = await _db.Customers
    //         .Where(c => id != null && c.CustomerSeq == id)
    //         .Include(c => c.User)
    //         .Include(c => c.Agreements)
    //         .ThenInclude(a => a.AgreementElements)
    //         .ThenInclude(ae => ae.ModuleLevel)
    //         .Include(c => c.Priority)
    //         .Include(c => c.Segment)
    //         .ThenInclude(s => s.MainSegment)
    //         .Include(c => c.Region)
    //         .AsSplitQuery()
    //         .ToListAsync();
    //     return res;
    //
    // }

    /// <summary>
    /// Read one Customer with id from database
    /// </summary>
    /// <param name="id"></param>
    /// <returns> Customer with corresponding id </returns>
    /// <return> Null if id not found </return>
    public async Task<Customer> GetCustomerById(int id)
    {
        var res = await _db.Customers.FindAsync(id);
        return res;
    }

    /// <summary>
    /// Adds a Customer to the database
    /// </summary>
    /// <param name="segment"></param>
    /// <returns> Customer Entity </returns>
    /// <returns> Null if a Customer is found with same CustomerName</returns>
    /// <exception cref="DbUpdateConcurrencyException"> Database: not updated database before new update </exception>
    public async Task<Customer> AddCustomer(Customer customer)
    {
        customer.CustomerSince = DateTime.Now;

        if (_db.Customers.Any(c => c.OrganizationNumber == customer.OrganizationNumber || c.CustomerName == customer.CustomerName))
            throw new InvalidOperationException($"OrgNr or CustomerName already Exist");

        // CapitalizeFirstLetter(customer.Region?.Area);
        // CapitalizeFirstLetter(customer.Region?.Country);

        var regionSeq = await GetRegionSeq(customer.Region);
        if (regionSeq != null)
        {
            customer.RegionSeq = regionSeq;
            customer.Region = null;
        }

        var res = _db.Customers.Add(customer);
        await _db.SaveChangesAsync();
        _db.ChangeTracker.Clear();
        
        return (await GetAllCustomers(res.Entity.CustomerSeq))[0];
    }

    /// <summary>
    /// Updates changes on a Customer in the database
    /// </summary>
    /// <param name="user"></param>
    /// <returns> The updated Customer entity </returns>
    /// <returns> Null if Customer with CustomerName already exist</returns>
    /// <exception cref="DbUpdateConcurrencyException"> Database: not updated database before new update </exception>
    public async Task<Customer> UpdateCustomer(Customer customer)
    {

        if (_db.Customers.Any(c => c.CustomerName == customer.CustomerName
        && c.CustomerSeq != customer.CustomerSeq))
            throw new InvalidOperationException($"Kunde med navn '{customer.CustomerName}' finnes allerede.");

        if (_db.Customers.Any(c => c.OrganizationNumber == customer.OrganizationNumber
                                   && c.CustomerSeq != customer.CustomerSeq))
            throw new InvalidOperationException($"Kunde med org.nr. '{customer.OrganizationNumber}' finnes allerede.");

        // CapitalizeFirstLetter(customer.Region?.Area);
        // CapitalizeFirstLetter(customer.Region?.Country);

        // Region checks
        // if (customer.RegionSeq != null)
        // {
        //     if (await CustomerRegionIsChanged(customer.RegionSeq, customer.Region))
        //     {
        //         var regionSeq = await GetRegionSeq(customer.Region);
        //         if (regionSeq != null)
        //         {
        //             customer.RegionSeq = regionSeq;
        //             customer.Region = null;
        //         }
        //     }
        // }
        // else
        // {
        //     var regionSeq = await GetRegionSeq(customer.Region);
        //     if (regionSeq != null)
        //     {
        //         customer.RegionSeq = regionSeq;
        //         customer.Region = null;
        //     }
        // }

        var isChanged = await CustomerRegionIsChanged(customer.RegionSeq, customer.Region);
        if (customer.RegionSeq != null && isChanged || customer.RegionSeq == null)
        {
            var regionSeq = await GetRegionSeq(customer.Region);
            if (regionSeq != null)
            {
                customer.RegionSeq = regionSeq;
                customer.Region = null;
            }
        }

        _db.Entry(customer).State = EntityState.Modified;

        await _db.SaveChangesAsync();
        _db.ChangeTracker.Clear();
        return (await GetAllCustomers(customer.CustomerSeq))[0];
    }

    /// <summary>
    /// Deactivated a Customer by setting IsActive attribute to false
    /// </summary>
    /// <param name="id"></param>
    /// <returns> Updated Customer </returns>
    /// <exception cref="NullReferenceException"> Thrown if id does not match a Customer id in the database </exception>
    public async Task<Customer> DeactivateCustomer(int id)
    {
        var customer = await _db.Customers.FindAsync(id);
        if (customer == null)
            throw new NullReferenceException($"Customer with id {id} doesn't exist");
        customer.IsActive = false;
        var res = await UpdateCustomer(customer);
        return res;
    }

    /// <summary>
    /// Activates a Customer by setting IsActive attribute to true
    /// </summary>
    /// <param name="id"></param>
    /// <returns> Updated Customer </returns>
    /// <exception cref="NullReferenceException"> Thrown if id does not match a Customer id in the database </exception>
    public async Task<Customer> ActivateCustomer(int id)
    {
        var customer = await _db.Customers.FindAsync(id);
        if (customer == null)
            throw new NullReferenceException($"Customer with id {id} doesn't exist");
        customer.IsActive = true;
        var res = await UpdateCustomer(customer);
        return res;
    }



    // private void CapitalizeFirstLetter(string? s)
    // {
    //     if (!string.IsNullOrEmpty(s)) s.CapitalizeFirstLetter();
    // }

    private async Task<bool> CustomerRegionIsChanged(int? regionSeq, Region region)
    {
        var oldRegion = await _db.Regions.FindAsync(regionSeq);
        if (oldRegion == null) return false;
        return !(oldRegion.Area == region.Area && oldRegion.Country == region.Country);
    }

    /// <summary>
    /// Checks in DB if it already exists a combination of area and country.
    /// </summary>
    /// <param name="region"></param>
    /// <returns>Returns primary key if combination exists. Not existing combination - creates new region entity and return primary key</returns>
    public async Task<int> GetRegionSeq(Region region)
    {
        // if (!string.IsNullOrEmpty(region.Area))
        // {
        //     var resAreaCountry = await _db.Regions.FirstOrDefaultAsync(r => r.Area == region.Area && r.Country == region.Country);
        //     return resAreaCountry?.RegionSeq;
        // }
        // var resCountry = await _db.Regions.FirstOrDefaultAsync(r => string.IsNullOrEmpty(r.Area) && r.Country == region.Country);
        // return resCountry?.RegionSeq;

        // if (string.IsNullOrEmpty(region.Area)) region.Area = null;

        if (string.IsNullOrEmpty(region.Area))
        {
            try
            {
                var res = await _db.Regions.Where(r => string.Equals(r.Country.ToLower(), region.Country.ToLower())).Where(r => r.Area == null).FirstAsync();
                Console.WriteLine(res.RegionSeq);
                return res.RegionSeq;
            }
            catch (InvalidOperationException e)
            {
                var newRegionObj = _db.Regions.Add(region);
                await _db.SaveChangesAsync();
                return newRegionObj.Entity.RegionSeq;
            }

        }


        try
        {
            var res = await _db.Regions.SingleAsync(r =>
                string.Equals(r.Area!.ToLower(), region.Area.ToLower()) &&
                string.Equals(r.Country.ToLower(), region.Country.ToLower()));
            return res.RegionSeq;
        }
        catch (InvalidOperationException e)
        {
            var newRegionObj = _db.Regions.Add(region);
            await _db.SaveChangesAsync();
            return newRegionObj.Entity.RegionSeq;
        }
    }

    }
}
