﻿using System;
using System.Collections.Generic;
using System.Linq;
using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Microsoft.EntityFrameworkCore;

namespace Escali.LicenseControl.DataAccess.Services
{

/// <summary>
/// MainSegmentService class inserts and updates MainSegment in the Database
/// </summary>
public class MainSegmentServices
{

    private readonly DataContext _db;

    public MainSegmentServices (DataContext db)
    {
        _db = db;
    }

    /// <summary>
    /// Read all Main Segements from database
    /// </summary>
    /// <returns> A List of Main Segements </returns>
    public async Task<List<MainSegment>> GetAllMainSegments()
    {
        var res = await _db.MainSegments.Include(ms => ms.Segments).ToListAsync();
        return res;
    }

    /// <summary>
    /// Read one Main Segement with id from database
    /// </summary>
    /// <param name="id"></param>
    /// <returns> Main Segement with corresponding id </returns>
    /// <return> Null if id not found </return>
    public async Task<MainSegment> GetMainSegmentById(int id)
    {
        var res = await _db.MainSegments.FindAsync(id);
        return res;
    }

    /// <summary>
    /// Adds a Main Segement to the database
    /// </summary>
    /// <param name="mainSegment"></param>
    /// <returns> Main Segement Entity </returns>
    /// <exception cref="DbUpdateException"> Database: error encountered while saving to database </exception>
    /// <exception cref="DbUpdateConcurrencyException"> Database: not updated database before new update </exception>
    public async Task<MainSegment> AddMainSegment(MainSegment mainSegment)
    {
        if (_db.MainSegments.Any(ms => ms.MainSegmentName == mainSegment.MainSegmentName))
            throw new InvalidOperationException($"Name already exist");

        var res = _db.MainSegments.Add(mainSegment);
        await _db.SaveChangesAsync();
        return res.Entity;
    }

    /// <summary>
    /// Updates changes on a Main Segement in the database
    /// </summary>
    /// <param name="mainSegment"></param>
    /// <returns> The updated Main Segement entity </returns>
    /// <exception cref="DbUpdateException"> Database: error encountered while saving to database </exception>
    /// <exception cref="DbUpdateConcurrencyException"> Database: not updated database before new update </exception>
    public async Task<MainSegment> UpdateMainSegment(MainSegment mainSegment)
    {
        if (_db.MainSegments.Any(ms => ms.MainSegmentName == mainSegment.MainSegmentName && ms.MainSegmentSeq != mainSegment.MainSegmentSeq))
            throw new InvalidOperationException($"Unique name already exist");

        var res = _db.MainSegments.Update(mainSegment);
        await _db.SaveChangesAsync();
        return res.Entity;
    }

    /// <summary>
    /// Deletes a Main Segement in the database
    /// </summary>
    /// <param name="id"></param>
    /// <returns> The deleted Main Segement entity </returns>
    /// <exception cref="DbUpdateException"> Database: error encountered while saving to database </exception>
    /// <exception cref="DbUpdateConcurrencyException"> Database: not updated database before new update </exception>
    public async Task<MainSegment> DeleteMainSegment(int id)
    {
        var found = await _db.MainSegments.FindAsync(id);
        if (found == null)
            return null;
        
        var mainSegment = _db.MainSegments.FindAsync(id);
        var res = _db.MainSegments.Remove(mainSegment.Result);
        await _db.SaveChangesAsync();
        return res.Entity;
    }

    }
}

