using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Microsoft.EntityFrameworkCore;

namespace Escali.LicenseControl.DataAccess.Services
{

/// <summary>
/// ModuleLevelService class inserts and updates ModuleLevel in the Database
/// </summary>
public class ModuleLevelService
{

    private readonly DataContext _db;

    public ModuleLevelService(DataContext db)
    {
        _db = db;
    }

    /// <summary>
    /// Read all ModuleLevels from database
    /// </summary>
    /// <returns> A List of ModuleLevels </returns>
    public async Task<List<ModuleLevel>> GetAllModuleLevels()
    {
        var res = await _db.ModuleLevels.ToListAsync();
        return res;
    }

    /// <summary>
    /// Read one ModuleLevel with id from database
    /// </summary>
    /// <param name="id"></param>
    /// <returns> ModuleLevel with corresponding id </returns>
    /// <returns> Null if ModuleLevel with id is not found</returns>
    public async Task<ModuleLevel> GetModuleLevelById(int id)
    {
        var res = await _db.ModuleLevels.FindAsync(id);
        return res;
    }

    /// <summary>
    /// Adds a ModuleLevel to the database
    /// </summary>
    /// <param name="moduleLevel"></param>
    /// <returns> Added ModuleLevel object </returns>
    /// <exception cref="DbUpdateException">Database error</exception>
    public async Task<ModuleLevel> AddModuleLevel(ModuleLevel moduleLevel)
    {
        if (_db.ModuleLevels.Any(m => m.ModuleLevelName == moduleLevel.ModuleLevelName && m.ModuleSeq == moduleLevel.ModuleSeq))
            throw new InvalidOperationException($"Modulelevel with name {moduleLevel.ModuleLevelName} already exists.");
        
        //if (moduleLevel.InvestmentLimit != null)
        //{
        //    var roundedInvestmentLimit = Math.Round(moduleLevel.InvestmentLimit.Value, 2);
        //    moduleLevel.InvestmentLimit = roundedInvestmentLimit;
        //}

        var res = _db.ModuleLevels.Add(moduleLevel);
        await _db.SaveChangesAsync();
        return res.Entity;
    }

    /// <summary>
    /// Updates changes on a ModuleLevel in the database
    /// </summary>
    /// <param name="moduleLevel"></param>
    /// <returns> Updated ModuleLevel object </returns>
    /// <exception cref="DbUpdateException">Database error</exception>
    /// <exception cref="DbUpdateConcurrencyException">Id doesn't exist</exception>
    public async Task<ModuleLevel> UpdateModuleLevel(ModuleLevel moduleLevel)
    {
        if (_db.ModuleLevels.Any(m => m.ModuleLevelName == moduleLevel.ModuleLevelName && m.ModuleSeq == moduleLevel.ModuleSeq && m.ModuleLevelSeq != moduleLevel.ModuleLevelSeq))
            throw new InvalidOperationException($"Modulelevel with name {moduleLevel.ModuleLevelName} already exists.");
        
        //if (moduleLevel.InvestmentLimit != null)
        //{
        //    var roundedInvestmentLimit = Math.Round(moduleLevel.InvestmentLimit.Value, 2);
        //    moduleLevel.InvestmentLimit = roundedInvestmentLimit;
        //}
        
        var res = _db.ModuleLevels.Update(moduleLevel);
        await _db.SaveChangesAsync();
        return res.Entity;
    }

    /// <summary>
    /// Activates a ModuleLevel by setting IsActive attribute to false
    /// </summary>
    /// <param name="id"></param>
    /// <returns> Updated ModuleLevel </returns>
    /// <exception cref="NullReferenceException"> Thrown if id does not match a ModuleLevel id in the database </exception>
    public async Task<ModuleLevel> DeactivateModuleLevel(int id)
    {
        var moduleLevel = await _db.ModuleLevels.FindAsync(id);
        if (moduleLevel == null) throw new NullReferenceException($"Module with id {id} doesn't exist.");
        moduleLevel.IsActive = false;
        var res = await UpdateModuleLevel(moduleLevel);
        return res;
    }

    /// <summary>
    /// Activates a ModuleLevel by setting IsActive attribute to false
    /// </summary>
    /// <param name="id"></param>
    /// <returns> Updated ModuleLevel </returns>
    /// <exception cref="NullReferenceException"> Thrown if id does not match a ModuleLevel id in the database </exception>
    public async Task<ModuleLevel> ActivateModuleLevel(int id)
    {
        var moduleLevel = await _db.ModuleLevels.FindAsync(id);
        if (moduleLevel == null) throw new NullReferenceException($"Module with id {id} doesn't exist.");
        moduleLevel.IsActive = true;
        var res = await UpdateModuleLevel(moduleLevel);
        return res;
    }

    }
}
