using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Microsoft.EntityFrameworkCore;

namespace Escali.LicenseControl.DataAccess.Services
{

/// <summary>
/// ModuleService class inserts and updates Module in the Database
/// </summary>
public class ModuleService
{
    
    private readonly DataContext _db;

    public ModuleService(DataContext db)
    {
        _db = db;
    }

    /// <summary>
    /// Read all Modules from database
    /// </summary>
    /// <returns> A List of Modules </returns>
    public async Task<List<Module>> GetAllModules()
    {
        var res = await _db.Modules
            .Include(m => m.ModuleLevels)
            .ToListAsync();
        return res;
    }

    /// <summary>
    /// Read one Module with id from database
    /// </summary>
    /// <param name="id"></param>
    /// <returns> Module with corresponding id </returns>
    /// <returns> Null if Module with id is not found</returns>
    public async Task<Module> GetModuleById(int id)
    {
        var res = await _db.Modules.FindAsync(id);
        return res;
    }

    /// <summary>
    /// Adds a Module to the database
    /// </summary>
    /// <param name="module"></param>
    /// <returns>Added module object</returns>
    /// <exception cref="DbUpdateException">Database error</exception>
    public async Task<Module> AddModule(Module module)
    {
        if (_db.Modules.Any(m => m.ModuleName == module.ModuleName && m.ProductSeq == module.ProductSeq))
            throw new InvalidOperationException($"Modul med navn '{module.ModuleName}' finnes allerede.");

        ModuleLevelDuplicateNamesCheck(module);
        
        var res = _db.Modules.Add(module);
        await _db.SaveChangesAsync();

        // var newModule = await _db.Modules
        //     .Where(m => m.ModuleSeq == res.Entity.ModuleSeq)
        //     .Include(m => m.ModuleLevels)
        //     .SingleOrDefaultAsync();
        //
        // if (newModule == null)
        // {
        //     throw new InvalidOperationException("Module ble ikke lagt til");
        // }
        _db.ChangeTracker.Clear();
        
        return res.Entity;
    }

    /// <summary>
    /// Updates changes on a Module in the database
    /// </summary>
    /// <param name="module"></param>
    /// <returns> Updated Module </returns>
    /// <exception cref="DbUpdateException"> Thrown if an error is encountered while saving to database </exception>
    /// <exception cref="DbUpdateConcurrencyException">Id doesn't exist</exception>
    public async Task<Module> UpdateModule(Module module)
    {
        if (_db.Modules.Any(m => m.ModuleName == module.ModuleName && m.ProductSeq == module.ProductSeq && m.ModuleSeq != module.ModuleSeq))
            throw new InvalidOperationException($"Modul med navn '{module.ModuleName}' finnes allerede.");
        
        ModuleLevelDuplicateNamesCheck(module);

        _db.Entry(module).State = EntityState.Modified;
        module.ModuleLevels.ForEach(ml => _db.Entry(ml).State = _db.Entry(ml).Entity.ModuleLevelSeq != 0 ? EntityState.Modified : EntityState.Added);
        
        await _db.SaveChangesAsync(); // TODO Lag metode for save and clear
        
        _db.ChangeTracker.Clear(); // Fixes tracking problems
        
        return module;
    }

    /// <summary>
    /// Deactivated a Module by setting IsActive attribute to false
    /// </summary>
    /// <param name="moduleSeq"></param>
    /// <returns> Updated Module </returns>
    /// <exception cref="NullReferenceException"> Thrown if moduleSeq does not match a Module id in the database </exception>
    public async Task<Module> DeactivateModule(int moduleSeq)
    {
        var module = await _db.Modules.FindAsync(moduleSeq);
        if (module == null) throw new NullReferenceException($"Module with id {moduleSeq} doesn't exist.");
        module.IsActive = false;
        var res = await UpdateModule(module);
        return res;
    }

    /// <summary>
    /// Activates a Module by setting IsActive attribute to true
    /// </summary>
    /// <param name="moduleSeq"></param>
    /// <returns> Updated Module </returns>
    /// <exception cref="NullReferenceException"> Thrown if moduleSeq does not match a Module id in the database </exception>
    public async Task<Module> ActivateModule(int moduleSeq)
    {
        var module = await _db.Modules.FindAsync(moduleSeq);
        if (module == null) throw new NullReferenceException($"Module with id {moduleSeq} doesn't exist.");
        module.IsActive = true;
        var res = await UpdateModule(module);
        return res;
    }

    /// <summary>
    /// Checks if modulelevelname exist in Modulelevel table in database
    /// </summary>
    /// <param name="module"></param>
    /// <exception cref="InvalidOperationException"></exception>
    private static void ModuleLevelDuplicateNamesCheck(Module module)
    {
        // Check for module level duplicate names
        if (module.ModuleLevels.GroupBy(ml => ml.ModuleLevelName).Any(item => item.Skip(1).Any()))
            throw new InvalidOperationException($"Modulnivåer kan ikke ha samme navn.");
    }

    }
}
