using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Microsoft.EntityFrameworkCore;

namespace Escali.LicenseControl.DataAccess.Services
{

/// <summary>
/// PriceElementService class inserts and updates PriceElement in the Database
/// </summary>
public class PriceElementService
{
    
    private readonly DataContext _db;

    public PriceElementService(DataContext db)
    {
        _db = db;
    }

    /// <summary>
    /// Read all PriceElements from database
    /// </summary>
    /// <returns> A List of PriceElements </returns>
    public async Task<List<PriceElement>> GetAllPriceElements()
    {
        var res = await _db.PriceElements
            .Include(pe => pe.PriceList)
            .ThenInclude(pl => pl.Currency)
            .ToListAsync();
        return res;
    }

    /// <summary>
    /// Read one PriceElement with id from database
    /// </summary>
    /// <param name="id"></param>
    /// <returns> PriceElement with corresponding id </returns>
    /// <returns> Null if PriceElement with id is not found</returns>
    public async Task<PriceElement> GetPriceElementById(int id)
    {
        var res = await _db.PriceElements.FindAsync(id);
        return res;
    }

    /// <summary>
    /// Adds a PriceElement to the database
    /// </summary>
    /// <param name="priceElement"></param>
    /// <returns> Added PriceElement object </returns>
    /// <exception cref="DbUpdateException">Database error</exception>
    public async Task<PriceElement> AddPriceElement(PriceElement priceElement)
    {
        var res = _db.PriceElements.Add(priceElement);
        await _db.SaveChangesAsync();
        return res.Entity;
    }

    /// <summary>
    /// Updates changes on a PriceElement in the database
    /// </summary>
    /// <param name="priceElement"></param>
    /// <returns> Updated PriceElement object </returns>
    /// <exception cref="DbUpdateException">Database error</exception>
    /// <exception cref="DbUpdateConcurrencyException">Id doesn't exist</exception>
    public async Task<PriceElement> UpdatePriceElement(PriceElement priceElement)
    {
        var res = _db.PriceElements.Update(priceElement);
        await _db.SaveChangesAsync();
        return res.Entity;
    }

    /// <summary>
    /// Deactivates a PriceElement by setting IsActive attribute to false
    /// </summary>
    /// <param name="id"></param>
    /// <returns> Updated PriceElement </returns>
    /// <exception cref="NullReferenceException"> Thrown if id does not match a PriceElement id in the database </exception>
    public async Task<PriceElement> DeactivatePriceElement(int id)
    {
        var priceElement = await _db.PriceElements.FindAsync(id);
        if (priceElement == null) throw new NullReferenceException($"PriceElement with id {id} doesn't exist.");
        priceElement.IsActive = false;
        var res = await UpdatePriceElement(priceElement);
        return res;
    }

    /// <summary>
    /// Activates a PriceElement by setting IsActive attribute to false
    /// </summary>
    /// <param name="id"></param>
    /// <returns> Updated PriceElement </returns>
    /// <exception cref="NullReferenceException"> Thrown if id does not match a PriceElement id in the database </exception>
    public async Task<PriceElement> ActivatePriceElement(int id)
    {
        var priceElement = await _db.PriceElements.FindAsync(id);
        if (priceElement == null) throw new NullReferenceException($"PriceElement with id {id} doesn't exist.");
        priceElement.IsActive = true;
        var res = await UpdatePriceElement(priceElement);
        return res;
    }

    }
}
