using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Utils;
using Microsoft.EntityFrameworkCore;

namespace Escali.LicenseControl.DataAccess.Services
{

/// <summary>
/// PriceListService class inserts and updates PriceList in the Database
/// </summary>
public class PriceListService
{
    
    private readonly DataContext _db;

    public PriceListService(DataContext db)
    {
        _db = db;
    }

    /// <summary>
    /// Read all PriceLists from database
    /// </summary>
    /// <returns> A List of PriceLists </returns>
    public async Task<List<PriceList>> GetAllPriceLists()
    {
        var res = await _db.PriceLists
            .Include(pl => pl.PriceElements)
            .Include(pl => pl.Currency)
            .ToListAsync();
        return res;
    }

    /// <summary>
    /// Read one PriceList with id from database
    /// </summary>
    /// <param name="id"></param>
    /// <returns> PriceList with corresponding id </returns>
    /// <returns> Null if PriceList with id is not found</returns>
    public async Task<PriceList> GetPriceListById(int id)
    {
        var res = await _db.PriceLists.FindAsync(id);
        return res;
    }
    
    /// <summary>
    /// Adds a PriceList to the database. Copies the PriceElements from priceListToCopy and multiply prices with priceAdjusmentFactor.
    /// </summary>
    /// <param name="priceList"></param>
    /// <returns> Added PriceList object </returns>
    /// <exception cref="InvalidOperationException">DateFrom isn't before DateTo</exception>
    /// <exception cref="DbUpdateException">Database error</exception>
    public async Task<PriceList> AddPriceListFromCopy(PriceList priceList, int priceListToCopy, decimal priceAdjustmentFactor)
    {
        if (_db.PriceLists.Any(p => p.PriceListName == priceList.PriceListName))
            throw new InvalidOperationException($"Prisliste med navn {priceList.PriceListName} finnes allerede.");
        
        if (priceList.DateFrom != null && priceList.DateTo != null)
            if (!ServiceUtils.DateFromIsBeforeDateTo(priceList.DateFrom.Value, priceList.DateTo.Value))
                throw new InvalidOperationException("Sluttdato må være etter startdato.");

        // priceList.CurrencySeq = await GetCurrencySeq(priceList.Currency.CurrencyName);
        // priceList.Currency = null;

        var res = _db.PriceLists.Add(priceList);
        await _db.SaveChangesAsync();
        var savedPriceList = res.Entity;
        
        var copy = await _db.PriceLists.Include(pl => pl.PriceElements).Where(pl => pl.PriceListSeq == priceListToCopy).ToListAsync();
        if (!copy.Any())
        {
            throw new InvalidOperationException("Prisliste for utgangspunkt finnes ikke i databasen.");
        }
        
        var copiedPriceElements = new List<PriceElement>();
        copy.First().PriceElements.ForEach(pe =>
        {
            var priceElementCopy = new PriceElement()
            {
                ModuleLevelSeq = pe.ModuleLevelSeq,
                Price = pe.Price * priceAdjustmentFactor,
                IsActive = pe.IsActive,
                PriceListSeq = savedPriceList.PriceListSeq
            };
            copiedPriceElements.Add(priceElementCopy);
        });

        savedPriceList.PriceElements = copiedPriceElements;
        await _db.SaveChangesAsync();

        return res.Entity;

    }

    /// <summary>
    /// Adds a PriceList to the database
    /// </summary>
    /// <param name="priceList"></param>
    /// <returns> Added PriceList object </returns>
    /// <exception cref="InvalidOperationException">DateFrom isn't before DateTo</exception>
    /// <exception cref="DbUpdateException">Database error</exception>
    public async Task<PriceList> AddPriceList(PriceList priceList)
    {
        if (_db.PriceLists.Any(p => p.PriceListName == priceList.PriceListName))
            throw new InvalidOperationException($"Prisliste med navn {priceList.PriceListName} finnes allerede.");
        
        if (priceList.DateFrom != null && priceList.DateTo != null)
            if (!ServiceUtils.DateFromIsBeforeDateTo(priceList.DateFrom.Value, priceList.DateTo.Value))
                throw new InvalidOperationException("Sluttdato må være etter startdato.");

        // priceList.CurrencySeq = await GetCurrencySeq(priceList.Currency.CurrencyName);
        // priceList.Currency = null;
        
        var res = _db.PriceLists.Add(priceList);
        await _db.SaveChangesAsync();
        return res.Entity;
    }

    /// <summary>
    /// Updates changes on a PriceList in the database
    /// </summary>
    /// <param name="priceList"></param>
    /// <returns> Updated PriceList object </returns>
    /// <exception cref="InvalidOperationException">DateFrom isn't before DateTo</exception>
    /// <exception cref="DbUpdateException">Database error</exception>
    public async Task<PriceList> UpdatePriceList(PriceList priceList)
    {
        if (_db.PriceLists.Any(p => p.PriceListName == priceList.PriceListName && p.PriceListSeq != priceList.PriceListSeq))
            throw new InvalidOperationException($"Prisliste med navn {priceList.PriceListName} finnes allerede.");
        
        if (priceList.DateFrom != null && priceList.DateTo != null)
            if (!ServiceUtils.DateFromIsBeforeDateTo(priceList.DateFrom.Value, priceList.DateTo.Value))
                throw new InvalidOperationException("Sluttdato må være etter startdato.");
        
        // priceList.CurrencySeq = await GetCurrencySeq(priceList.Currency.CurrencyName);
        // priceList.Currency = null;

        _db.Entry(priceList).State = EntityState.Modified;
        await _db.SaveChangesAsync();
        _db.ChangeTracker.Clear();
        
        priceList.Currency = await _db.Currencies.FindAsync(priceList.CurrencySeq);
        
        return priceList;
    }

    }
}
