using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Microsoft.EntityFrameworkCore;

namespace Escali.LicenseControl.DataAccess.Services
{

/// <summary>
/// ProductService class inserts and updates Product in the Database
/// </summary>
public class ProductService
{
    private readonly DataContext _db;

    public ProductService(DataContext db)
    {
        _db = db;
    }

    /// <summary>
    /// Read all Products from database
    /// </summary>
    /// <returns> A List of Products </returns>
    public async Task<List<Product>> GetAllProducts()
    {
        var res = await _db.Products.ToListAsync();
        return res;
    }

    /// <summary>
    /// Read all Products from database included referenced tables
    /// </summary>
    /// <returns>A List of Products</returns>
    public async Task<List<Product>> GetAllProductWithChildren()
    {
        var res = await _db.Products
            .Include(p => p.PriceLists)
            .Include(p => p.Modules)
            .ThenInclude(m => m.ModuleLevels)
            .ThenInclude(ml => ml.PriceElements)
            .ThenInclude(pe => pe.PriceList)
            .ThenInclude(pl => pl.Currency)
            .Include(p => p.Agreements)
            .ThenInclude(a => a.AgreementElements)
            .AsSplitQuery()
            .ToListAsync();
        return res;
    }

    /// <summary>
    /// Read one Product with id from database
    /// </summary>
    /// <param name="id"></param>
    /// <returns> Product with corresponding id </returns>
    /// <returns> Null if Product with id is not found</returns>
    public async Task<Product> GetProductById(int id)
    {
        var res = await _db.Products.FindAsync(id);
        return res;
    }

    /// <summary>
    /// Adds a Product to the database
    /// </summary>
    /// <param name="product"></param>
    /// <returns> Product Entity </returns>
    /// <exception cref="DbUpdateException">Database error</exception>
    public async Task<Product> AddProduct(Product product)
    {
        if (_db.Products.Any(p => p.ProductName == product.ProductName))
            throw new InvalidOperationException($"Unique name violation.");
        
        var res = _db.Products.Add(product);
        await _db.SaveChangesAsync();
        return res.Entity;

    }

    /// <summary>
    /// Updates changes on a Product in the database
    /// </summary>
    /// <param name="product"></param>
    /// <returns>The updated Product entity </returns>
    /// <exception cref="DbUpdateException">Database error</exception>
    /// <exception cref="DbUpdateConcurrencyException">Id doesn't exist</exception>
    public async Task<Product> UpdateProduct(Product product)
    {
        if (_db.Products.Any(p => p.ProductName == product.ProductName && p.ProductSeq != product.ProductSeq))
            throw new InvalidOperationException($"Unique name violation.");
        
        var res = _db.Products.Update(product);
        await _db.SaveChangesAsync();
        return res.Entity;
    }

    }
}
